package aln.spawncommands;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.command.CommandException;
import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

public class CommandAway implements ICommand {
  
  @Override
  public int compareTo(ICommand arg0) {
    return 0;
  }
  
  @Override
  public String getCommandName() {
    return Do.getConfigArrayValue(SpawnCommands.commandNames ,"away");
  }
  
  @Override
  public String getCommandUsage(ICommandSender sender) {
    return "/"+ Do.getConfigArrayValue(SpawnCommands.commandNames ,"away") +" [<block-distance>]     Teleports you somewhat randomly far away from where you are now.  This command has a usage cool down time of "+ Do.getConfigArrayValueInteger(SpawnCommands.miscConfigArray,"away coolDownMinutes") +" minutes.";
  }
  
  @Override
  public List<String> getCommandAliases() {
    return Arrays.asList(new String[] { });
  }
  
  @Override
  public void execute(MinecraftServer server, ICommandSender sender, String[] args) throws CommandException {
    if(sender instanceof EntityPlayer) {
      EntityPlayer player = (EntityPlayer) sender;
      String playerName = player.getName();
      World world = player.worldObj;
      if (world.isRemote) { return; }
      
      // get these from the miscConfig file
      SpawnCommands.loadMiscConfig();
      double distanceAway = Do.getConfigArrayValueDouble(SpawnCommands.miscConfigArray,"away distanceAway");
      double distanceVariance = Do.getConfigArrayValueDouble(SpawnCommands.miscConfigArray,"away distanceVariance");
      double minDistanceFromWorldSpawn = Do.getConfigArrayValueDouble(SpawnCommands.miscConfigArray,"away minDistanceFromWorldSpawn");
      double minDistanceFromSelf = Do.getConfigArrayValueDouble(SpawnCommands.miscConfigArray,"away minDistanceFromSelf");
      double maxDistanceFromSelf = Do.getConfigArrayValueDouble(SpawnCommands.miscConfigArray,"away maxDistanceFromSelf");
      int coolDownMinutes = Do.getConfigArrayValueInteger(SpawnCommands.miscConfigArray,"away coolDownMinutes");
      
      String lastUsePathAndFileName = SpawnCommands.configBasePath + playerName +"/_timeLastUsedCommandAway";
      
      long timeNow = (new Date()).getTime(); // in milliseconds
      String timeNowString = Long.toString(timeNow);
      
      if ( ( args.length > 1) || ( args.length == 1 ) && ( args[0].equalsIgnoreCase("help")) ) { Do.Say(player,getCommandUsage(sender)); return; }
      
      // command permissions. "no" is handled by not registering the command at all. "op" and "enable" are handled here. "all" is the default.
      SpawnCommands.loadCommandPermissionsConfig();
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "away").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"Op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "away").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"Advanced spawn commands are not enabled."); return; }
      
      // check command cool down time
      if (( coolDownMinutes > 0 ) && (! Do.IsOp(player)) ) { // allow ops to do this command as much as they like.
        long lastUse;
        try {
          lastUse = Long.parseLong(Do.FileToString(lastUsePathAndFileName));
        } catch (NumberFormatException e) { lastUse = 0; }
        long timeLeft =(lastUse + (coolDownMinutes*60*1000)) - timeNow; // in milliseconds
        long timeLeftMinutes = (timeLeft/1000/60) + 1; // the +1 is to prevent a display of 0 remaining time when there is less than a minute left. 
        long timeLeftHours = (timeLeftMinutes/60);
        if (timeLeft > 0 ) {
          Do.Say(player,"The /"+Do.getConfigArrayValue(SpawnCommands.commandNames ,"away")+" command is still on cool down");
          Do.Say(player,"   for another " +((int)timeLeftHours)+" hours and "+((int)( timeLeftMinutes - (((int)timeLeftHours)*60) ))+" minutes.");
          return;
        }
      }
      
      double newX = 0;
      double newY = 0;
      double newZ = 0;
      
      // parse distance parameter to a number
      if ( args.length > 0) {
        try {
          distanceAway = Double.parseDouble(args[0]);
        } catch (NumberFormatException e) {
          Do.Say(player,"That was not a number.  Please specify a distance in number of blocks.  If you do not specify then "+ (int)distanceAway +" will be used.");
          return;
        }
      }
      // error check value
      if ( distanceAway < 0 ) { distanceAway = -distanceAway; } // no negative numbers
      if ( distanceAway < minDistanceFromSelf ) { // limits 
        distanceAway = minDistanceFromSelf;
        Do.Say(player,"Too close to yourself.  Using "+(int)distanceAway+" instead.");
      }
      if ( distanceAway > maxDistanceFromSelf ) { // limits
        distanceAway = maxDistanceFromSelf;
        Do.Say(player,"Too far away.  Using " + (int)distanceAway +" instead.");
      }
      
      // attempt max arbitrary times to find a new random location that is not near main spawn
      boolean foundASpot = false;
      int tryNumber;
      for(tryNumber=1;tryNumber < 20;tryNumber++) { 

        //pick random coordinate in range
        Random rnd = new Random();
        double newVariance = ( rnd.nextDouble() * (distanceVariance*2) ) - distanceVariance;
        distanceAway += newVariance;
        double newAngleRadians = (rnd.nextDouble() * Math.PI*2); // info: pi*2 == number of Radians in a circle.
        newX = (distanceAway * Math.cos(newAngleRadians)) + Math.round(player.posX - .5);
        newZ = (distanceAway * Math.sin(newAngleRadians)) + Math.round(player.posZ - .5); // Thanks Ron on my "brain no math" day.
        
        // if not main world dont worry about distance to main spawn
        if ( player.dimension != 0 ) { foundASpot = true; break; } 
        
        // if not too close to main spawn we found a spot
        if (!( Do.distance2d(newX, newZ, (double) world.getSpawnPoint().getX(), (double) world.getSpawnPoint().getZ()) < minDistanceFromWorldSpawn )) {
          foundASpot = true;
          break;
        }
        
      }
      if ( ! foundASpot ) { Do.Say(player,"Sorry, tried "+ tryNumber +" of times to find a new location and failed.  Try again or check "+ ModInfo.NAME +" config files."); return; }
      
      // choose a height (y coordinate)
      newY = world.getActualHeight() + 1; // start looking at this height
      foundASpot = false;
      boolean topIsBedrock = false;
      if ( Do.blockModAndName(player,newX,world.getActualHeight()-1,newZ).equalsIgnoreCase("minecraft:bedrock") ) {
        // this dimension might have a bedrock ceiling like the nether so...  look down for place to spawn
        topIsBedrock = true;
        while( newY > 1) {
          newY--;
          if ( ( (SpawnCommands.canSpawnInsideBlock(player, newX, newY, newZ)) && (SpawnCommands.canSpawnInsideBlock(player, newX, newY + 1, newZ)) ) ) {
            foundASpot = true;
            break;
          }
        }
      } // end if top block at y is bedrock
      if ( ( topIsBedrock ) && ( ! foundASpot ) ) { Do.Say(player,"Sorry, faild to find a new location not inside solid blocks.  Try again."); return; }
      
      // check if no bottom and will fall through to the void; consider the end, or void worlds or skyblock maps
      if (( newY <= 1 ) && ( SpawnCommands.canSpawnInsideBlock(player, newX, newY - 1, newZ) )) {
        Do.Say(player,"Sorry, failed to find a new location with a solid block to stand on.  Try again.");
        return;
      }
      
      // we have now successfully found a place to send the player
      
      // record when they last used the /away command (now) for command cool down time
      Do.StringToFile(lastUsePathAndFileName,timeNowString);
      
      Do.TeleportPlayerToSC(player, newX, newY, newZ, player.rotationYaw, player.rotationPitch, player.dimension);
      
      Do.Say(player,"You are now \u00A7e"+ (int)Do.distance2d(Math.round(player.posX - .5), Math.round(player.posZ - .5), newX, newZ) +" blocks away\u00A7r from where you were.");
      Do.Say(player,"Slow load times can drop you underground and may kill you."); // thank you Mojang
      Do.Say(player,"You may have to wait for the world to appear around you.");
      
      return;
    }
  }
  
  @Override
  public boolean checkPermission(MinecraftServer server, ICommandSender sender) {
    return true;
  }
  
  @Override
  public List<String> getTabCompletionOptions(MinecraftServer server, ICommandSender sender, String[] args, BlockPos pos) {
    return null;
  }
  
  @Override
  public boolean isUsernameIndex(String[] args, int index) {
    return false;
  }
  
}
