package aln.spawncommands;

import java.util.Arrays;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.command.CommandException;
import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Blocks;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

public class CommandDown implements ICommand {
  
  @Override
  public int compareTo(ICommand arg0) {
    return 0;
  }
  
  @Override
  public String getCommandName() {
    return Do.getConfigArrayValue(SpawnCommands.commandNames ,"dn");
  }
  
  @Override
  public String getCommandUsage(ICommandSender sender) {
    return "/"+ Do.getConfigArrayValue(SpawnCommands.commandNames ,"dn") +"     Teleports you down to the next lower place you can stand at this x and z coordinate.";
  }
  
  @Override
  public List<String> getCommandAliases() {
    return Arrays.asList(new String[] { });
  }
  
  @Override
  public void execute(MinecraftServer server, ICommandSender sender, String[] args) throws CommandException {
    if(sender instanceof EntityPlayer) {
      EntityPlayer player = (EntityPlayer) sender;
      World world = player.worldObj;
      if (world.isRemote) { return; }
      
      // command permissions. "no" is handled by not registering the command at all. "op" and "enable" are handled here. "all" is the default.
      SpawnCommands.loadCommandPermissionsConfig();
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "dn").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"Op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "dn").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"Advanced spawn commands are not enabled."); return; }
      
      // get player's location at start of this teleport request
      double  px = Math.round(player.posX - .5); // player's coordinates rounded down
      double  py = Math.round(player.posY - .5);
      double  pz = Math.round(player.posZ - .5);
      Float   pyaw = player.rotationYaw;
      Float   ppitch = player.rotationPitch;
      Integer pdim = world.provider.getDimension(); // mc1.9
      String newGoBackLocation = px +","+ py +","+ pz +","+ pyaw +","+ ppitch +","+ pdim;
      
      // find the next acceptable location to stand below here
      boolean foundSolidBelow = false;
      boolean foundGoodLocation = false;
      py--; // start looking for solid 1 block below the player (they might be flying or on the edge of something now)
      
      // if air/space right below the player then land on the next solid block below that 
      // ( think already flying or hanging off the edge of a cliff )
      if ( SpawnCommands.canSpawnInsideBlock(player, px, py, pz) ) { foundSolidBelow = true; } 
      
      while ( (py >= 1 ) && ( (! foundSolidBelow) || (! foundGoodLocation) ) ) {
        if ( (! foundSolidBelow) && ( ! SpawnCommands.canSpawnInsideBlock(player, px, py, pz) ) ) {
          foundSolidBelow = true;
          py = py - 2; // down 3 because this spot is blocked and we need 2 open spots for the player below this 
        }
        if ( (foundSolidBelow) && ( (SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) && (SpawnCommands.canSpawnInsideBlock(player, px, py+1, pz)) ) ) {
          foundGoodLocation = true;
          break;
        }
        py--; // check next block down
      }
      if ( ! foundGoodLocation ) { Do.Say(player,"You are at the lowest point."); return; } else { Do.Say(player,"Down!"); }
      
      // falling check
      while (  ( py > 1 ) && ( (SpawnCommands.canSpawnInsideBlock(player, px, py - 1, pz)) && (SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) ) ) 
      { py--; }
      
      BlockPos posBelowFeet = new BlockPos(px, (py-1), pz);
      Block blockBelowFeet = player.worldObj.getBlockState(posBelowFeet).getBlock();
      
      // if at y==1 then check for no bedrock under feet 
      if ( (py == 1) && (blockBelowFeet != Blocks.BEDROCK)) { 
        player.worldObj.setBlockState(posBelowFeet, Blocks.COBBLESTONE.getDefaultState()); 
      }
      
      // lava under feet check // mc1.8,mc1.9
      if ((py > 1) && ( (blockBelowFeet == Blocks.LAVA) || (blockBelowFeet == Blocks.FLOWING_LAVA))) {
        player.worldObj.setBlockState(posBelowFeet, Blocks.COBBLESTONE.getDefaultState());
      }
      // mc1.7.10
      //if ((py>1) && ( ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.lava) || ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.flowing_lava) )) {
      //  player.worldObj.setBlock((int)px, (int)(py-1), (int)pz, Blocks.cobblestone);
      //}
      
      // actually go there now
      //((EntityPlayerMP) player).playerNetServerHandler.setPlayerLocation(px + .5d, py, pz + .5d, pyaw, ppitch);
      //player.setPositionAndUpdate(px + .5d, py, pz + .5d);
      Do.TeleportPlayerTo(player, px, py, pz);
      
      // write the location of the next go back command
      if (! Do.StringToFile( SpawnCommands.configBasePath + player.getName() + "/_" + player.getName() +"_GoBackLocation", newGoBackLocation ) ) // mc1.8 
      { Do.Err(player,"CommandSpawn: Could not create the file that stores the location to go back to."); return; }

      return;
      
    }
  }
  
  @Override
  public boolean checkPermission(MinecraftServer server, ICommandSender sender) {
    return true;
  }
  
  @Override
  public List<String> getTabCompletionOptions(MinecraftServer server, ICommandSender sender, String[] args, BlockPos pos) {
    return null;
  }
  
  @Override
  public boolean isUsernameIndex(String[] args, int index) {
    return false;
  }
  
}
