package aln.spawncommands;

import java.util.Arrays;
import java.util.List;

import net.minecraft.command.CommandException;
import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.registry.GameData;

public class CommandSpawnBlockData implements ICommand {
  
  @Override
  public int compareTo(ICommand arg0) {
    return 0;
  }
  
  @Override
  public String getCommandName() {
    return Do.getConfigArrayValue(SpawnCommands.commandNames ,"spawnblockdata");
  }
  
  @Override
  public String getCommandUsage(ICommandSender sender) {
    return "/"+ Do.getConfigArrayValue(SpawnCommands.commandNames ,"spawnblockdata") +"   displays data about the blocks you are standing in and on.";
  }
  
  @Override
  public List getCommandAliases() {
    return Arrays.asList(new String[] { });
  }
  
  @Override
  public void execute(MinecraftServer server, ICommandSender sender, String[] params) throws CommandException {
    if (!(sender instanceof EntityPlayer)) { return; }
    EntityPlayer player = (EntityPlayer) sender;
    World world = player.worldObj;
    if ( world.isRemote ) { return; }
    
    // command permissions. "no" is handled by not registering the command at all. "op" and "enable" are handled here. "all" is the default.
    SpawnCommands.loadCommandPermissionsConfig();
    if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawnblockdata").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
    { Do.Say(player,"Op only command.  You are not an op."); return; }
    if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawnblockdata").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
    { Do.Say(player,"Advanced spawn commands are not enabled."); return; }
    
    if ( (params.length > 0) && (params[0].equalsIgnoreCase("help")) ) { SpawnCommands.showHelp(player); return; }
    Do.Say(player, " ");
    
    // mc1.8
    //double  px = Math.round(player.getPosition().getX() - .5); // player's coordinates rounded down
    //double  py = Math.round(player.getPosition().getY() - .5);
    //double  pz = Math.round(player.getPosition().getZ() - .5);
    //mc 1.7.10
    double  px = Math.round(player.posX - .5); // player's coordinates rounded down
    double  py = Math.round(player.posY - .5);
    double  pz = Math.round(player.posZ - .5);
      
    py--; // targeting the block below feet
    
    if ( py < 0 ) { Do.Say(player, "There is nothing below your feet."); return; }
    if ( py > world.getActualHeight() ) { Do.Say(player, "You are above the sky limit for blocks."); return; }
    
    // mc1.8 mc1.9
    Do.Say(player,"\u00A77Dimension name "+world.provider.getDimensionType().getName()+"\u00A7r"); // mc1.9
    //Do.Say(player,"\u00A78Dimension name "+world.provider.getDimensionName()+"\u00A7r"); // mc1.8
    // example:   Railcraft:tile.railcraft.residual.heat
    
    // mc1.8
    BlockPos pos = new BlockPos(px,py,pz);
    Do.Say(player, " ");
    Do.Say(player, "Block at x"+ (int)px +" y"+ (int)py +" z"+ (int)pz +" \u00A7ebelow player\u00A7r "+ player.getName());
    //Do.Say(player, "Block name "+ GameData.getBlockRegistry().getNameForObject(world.getBlockState(pos).getBlock()) +" with meta data "+ player.worldObj.getChunkFromBlockCoords(pos).getBlockMetadata(pos) ); // mc1.8
    Do.Say(player,"\u00A72"+ Do.blockModName(player, pos)+"\u00A7c:\u00A72"+Do.blockName(player, pos)+"\u00A7r"+(Do.blockMetadataString(player, pos).length() > 0 ? "," : "")+"\u00A7e"+Do.blockMetadataString(player, pos)+"\u00A7r");
    if ( world.getBlockState(pos).getBlock().hasTileEntity(world.getBlockState(pos)) ) { Do.Say(player, "\u00A78Block is a tileEntity.\u00A7r"); } 
    // mc1.7.10
    //Do.Say(player,"\u00A78Dimension name "+world.provider.getDimensionName()+"\u00A7r");
    //Do.Say(player, "Block at x"+ (int)px +" y"+ (int)py +" z"+ (int)pz +" \u00A7ebelow player\u00A7r "+ player.getGameProfile().getName());
    //Do.Say(player, "Block name "+ GameData.getBlockRegistry().getNameForObject(world.getBlock((int)px,(int)py,(int)pz)) +" with meta data "+ world.getBlockMetadata((int)px,(int)py,(int)pz) );
    //Do.Say(player, "\u00A78"+world.getBlock((int)px,(int)py,(int)pz).toString()+"\u00A7r");
    //if ( world.getBlock((int)px,(int)py,(int)pz).hasTileEntity( world.getBlockMetadata((int)px,(int)py,(int)pz) ) ) { Do.Say(player, "\u00A78Block is a tileEntity.\u00A7r"); } 
    
    // mc1.8
    pos = new BlockPos(px,py+1,pz);
    Do.Say(player, " ");
    Do.Say(player, "Block at x"+ (int)px +" y"+ (int)py +" z"+ (int)pz +" \u00A7eat player's feet\u00A7r");
    //Do.Say(player, "Block name "+ GameData.getBlockRegistry().getNameForObject(world.getBlockState(pos).getBlock()) +" with meta data "+ player.worldObj.getChunkFromBlockCoords(pos).getBlockMetadata(pos) ); // mc1.8
    Do.Say(player,"\u00A72"+ Do.blockModName(player, pos)+"\u00A7c:\u00A72"+Do.blockName(player, pos)+"\u00A7r"+(Do.blockMetadataString(player, pos).length() > 0 ? "," : "")+"\u00A7e"+Do.blockMetadataString(player, pos)+"\u00A7r");
    if ( world.getBlockState(pos).getBlock().hasTileEntity(world.getBlockState(pos)) ) { Do.Say(player, "\u00A78Block is a tileEntity.\u00A7r"); } 
    // mc1.7.10
    //Do.Say(player,"\u00A78Dimension name "+world.provider.getDimensionName()+"\u00A7r");
    //Do.Say(player, "Block at x"+ (int)px +" y"+ (int)py +" z"+ (int)pz +" \u00A7ebelow player\u00A7r "+ player.getGameProfile().getName());
    //Do.Say(player, "Block name "+ GameData.getBlockRegistry().getNameForObject(world.getBlock((int)px,(int)py,(int)pz)) +" with meta data "+ world.getBlockMetadata((int)px,(int)py,(int)pz) );
    //Do.Say(player, "\u00A78"+world.getBlock((int)px,(int)py,(int)pz).toString()+"\u00A7r");
    //if ( world.getBlock((int)px,(int)py,(int)pz).hasTileEntity( world.getBlockMetadata((int)px,(int)py,(int)pz) ) ) { Do.Say(player, "\u00A78Block is a tileEntity.\u00A7r"); } 
    
    // mc1.8
    pos = new BlockPos(px,py+2,pz);
    Do.Say(player, " ");
    Do.Say(player, "Block at x"+ (int)px +" y"+ (int)py +" z"+ (int)pz +" \u00A7eat player's head\u00A7r");
    //Do.Say(player, "Block name "+ GameData.getBlockRegistry().getNameForObject(world.getBlockState(pos).getBlock()) +" with meta data "+ player.worldObj.getChunkFromBlockCoords(pos).getBlockMetadata(pos) ); // mc1.8
    Do.Say(player,"\u00A72"+ Do.blockModName(player, pos)+"\u00A7c:\u00A72"+Do.blockName(player, pos)+"\u00A7r"+(Do.blockMetadataString(player, pos).length() > 0 ? "," : "")+"\u00A7e"+Do.blockMetadataString(player, pos)+"\u00A7r");
    if ( world.getBlockState(pos).getBlock().hasTileEntity(world.getBlockState(pos)) ) { Do.Say(player, "\u00A78Block is a tileEntity.\u00A7r"); } 
    // mc1.7.10
    //Do.Say(player,"\u00A78Dimension name "+world.provider.getDimensionName()+"\u00A7r");
    //Do.Say(player, "Block at x"+ (int)px +" y"+ (int)py +" z"+ (int)pz +" \u00A7ebelow player\u00A7r "+ player.getGameProfile().getName());
    //Do.Say(player, "Block name "+ GameData.getBlockRegistry().getNameForObject(world.getBlock((int)px,(int)py,(int)pz)) +" with meta data "+ world.getBlockMetadata((int)px,(int)py,(int)pz) );
    //Do.Say(player, "\u00A78"+world.getBlock((int)px,(int)py,(int)pz).toString()+"\u00A7r");
    //if ( world.getBlock((int)px,(int)py,(int)pz).hasTileEntity( world.getBlockMetadata((int)px,(int)py,(int)pz) ) ) { Do.Say(player, "\u00A78Block is a tileEntity.\u00A7r"); } 
  }
  
  @Override
  public boolean checkPermission(MinecraftServer server, ICommandSender icommandsender) { // mc1.8
  //public boolean canCommandSenderUseCommand(ICommandSender icommandsender) { // mc1.7.10
    return true;
  }
  
  @Override
  public List getTabCompletionOptions(MinecraftServer server, ICommandSender sender, String[] args, BlockPos pos) { // mc1.8
  //public List addTabCompletionOptions(ICommandSender icommandsender, String[] astring) { // mc1.7.10
    return null;
  }
  
  @Override
  public boolean isUsernameIndex(String[] astring, int i) {
    return false;
  }

}
