package aln.spawncommands;

import java.util.Arrays;
import java.util.List;

import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.DimensionType;
import net.minecraft.world.World;

public class CommandSpawnQuery implements ICommand {

  @Override
  public int compareTo(ICommand o) {
    return 0;
  }
  
  @Override
  public String getCommandName() {
    //return "spawn?";
	return Do.getConfigArrayValue(SpawnCommands.commandNames ,"spawn?");
  }
  
  @Override
  public String getCommandUsage(ICommandSender icommandsender) {
    return "/"+ Do.getConfigArrayValue(SpawnCommands.commandNames ,"spawn?") +" - displays with world and bed spawn coordinates.";
  }
  
  @Override
  public List getCommandAliases() {
    return Arrays.asList(new String[] { });
  }
  
  @Override
  public void execute(MinecraftServer server, ICommandSender icommandsender, String[] params) {
    if(icommandsender instanceof EntityPlayer) {
      EntityPlayer player = (EntityPlayer) icommandsender;
      World world = player.worldObj;
      if ( world.isRemote ) { return; }
      
      SpawnCommands.loadCommandPermissionsConfig();
      SpawnCommands.loadMiscConfig();
      
      // command permissions. "no" is handled by not registering the command at all. "op" and "enable" are handled here. "all" is the default.
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn?").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"Op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn?").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"Advanced spawn commands are not enabled."); return; }
      
      if (params.length > 0) { SpawnCommands.showHelp(player); return; }
      
      int originDim = player.dimension; // mc1.9.4
      //int originDim = world.provider.getDimension(); // mc1.9
      //int originDim = world.provider.dimensionId; // mc1.7.10
      //int originDim = world.provider.getDimensionId(); // mc1.8
      int bedDim;
      boolean bedIsInThisDim = false;
      //ChunkCoordinates bedCC = null; // mc1.7.10
      BlockPos bedPos = null; // mc1.8
      
      Do.Say(player," ");
      
      Do.Say(player,player.getName() // mc1.8
      //Do.Say(player,player.getGameProfile().getName() // mc1.7.10
        +" is at ( x"
        +Math.round(player.posX - .5)+" y"
        +Math.round(player.posY - .5)+" z"
        +Math.round(player.posZ - .5)+" )"
        );
      Do.Say(player,"   in dimension ("+player.dimension+") "+DimensionType.getById(player.dimension).getName() +" ");

      // is there a bed location defined in the local dimension?
      //bedCC = player.getBedLocation(originDim); // mc1.7.10
      //if ( bedCC == null ) { bedIsInThisDim = false; } else { bedIsInThisDim = true; } // mc1.7.10
      bedPos = player.getBedLocation(originDim); // mc1.8
      if ( bedPos == null ) { bedIsInThisDim = false; } else { bedIsInThisDim = true; } // mc1.8
      if ( bedIsInThisDim ) { bedDim = originDim; } else { bedDim = 0; }
      if (  Do.isModIDLoaded("TwilightForest") 
          //&& (world.provider.getDimensionName().equals("Twilight Forest")) // mc1.8.9
          && (world.provider.getDimensionType().getName().equals("Twilight Forest")) // mc1.9 
          && bedIsInThisDim 
         )
      { bedDim = 0; bedIsInThisDim = false; } // exception for TwilightForest mod
      
      // mc1.7.10
      //if ( ( originDim != 0) && (bedDim == originDim) && (bedCC != null) ) {
      //  Do.Say(player,"Bed/home in this dimension is at ( x"+bedCC.posX+" y"+bedCC.posY+" z"+bedCC.posZ+" )" );
      //}
      // mc1.8
      if ( ( originDim != 0) && (bedDim == originDim) && (bedPos != null) ) {
        Do.Say(player,"Bed/home in this dimension is at ( x"+bedPos.getX()+" y"+bedPos.getY()+" z"+bedPos.getZ()+" )" );
      }
      
      // mc 1.8
      BlockPos pxyz = player.worldObj.getSpawnPoint(); // gets from provider worldInfo
      double px = pxyz.getX();
      double py = pxyz.getY();
      double pz = pxyz.getZ();
      // mc 1.7.10
      //ChunkCoordinates pxyz = player.worldObj.getSpawnPoint();
      //double px = pxyz.posX;
      //double py = pxyz.posY;
      //double pz = pxyz.posZ;
      Do.Say(player,"Main world spawn coordinates ( x" + (int)px + " y" + (int)py + " z" + (int)pz + " )");
      bedPos = player.getBedLocation(0); // mc1.8 
      if ( bedPos != null) { // mc1.8
        Do.Say(player,"Main world bed/home spawn point ( x" + (int)bedPos.getX() +" y"+ (int)bedPos.getY() +" z"+ (int)bedPos.getZ() + " )"); // mc1.8
      //bedCC = player.getBedLocation(0); // mc1.7.10
      //if ( bedCC != null) { // mc1.7.10
      //  Do.Say(player,"Main world bed/home spawn point ( x" + (int)bedCC.posX +" y"+ (int)bedCC.posY +" z"+ (int)bedCC.posZ + " )"); // mc1.7.10
      } else {
        Do.Say(player,"Main world bed/home spawn point is not set.");
      }
    }
  }

  @Override
  public boolean checkPermission(MinecraftServer server, ICommandSender icommandsender) { // mc1.8
  //public boolean canCommandSenderUseCommand(ICommandSender icommandsender) { // mc1.7.10
    return true;
  }
  
  @Override
  public List getTabCompletionOptions(MinecraftServer server, ICommandSender sender, String[] args, BlockPos pos) { // mc1.8
  //public List addTabCompletionOptions(ICommandSender icommandsender, String[] astring) { // mc1.7.10
    return null;
  }
  
  @Override
  public boolean isUsernameIndex(String[] astring, int i) {
    return false;
  }
  
}
