package aln.spawncommands;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import net.minecraft.command.CommandBase;
import net.minecraft.command.CommandException;
import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.Entity;
import net.minecraft.entity.boss.EntityDragon;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Blocks;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

// public class CommandTeleportAsk implements ICommand { // mc1.8.9
public class CommandTeleportInvite  extends CommandBase {
  // This command is like /ta but in reverse. It invites a player to come to you instead of asking to go to them.
  
  private static final String TAConfigBasePath = SpawnCommands.configBasePath+"/_TeleportAsk/";
  public static final String lastPlayerAskedFileName = "_TeleportAsk_LastPlayerAsked"; // see CommandTeleportAsk
  public static final String incomingResponseFileName = "_TeleportAsk_IncomingResponse.txt";

  
  @Override
  public int compareTo(ICommand arg0) {
    return 0;
  }
  
  @Override
  public String getCommandName() {
    return Do.getConfigArrayValue(SpawnCommands.commandNames ,"ti");
  }
  
  @Override
  public String getCommandUsage(ICommandSender sender) { 
    // Apparently this is run EVERY time someone does /help of any kind...
    //return "/ta notch -Asks notch if you may teleport to him. Then telports you to Sweeden.Ya!"; // limited to 82 characters long lol
    return "/"+ Do.getConfigArrayValue(SpawnCommands.commandNames ,"ti") +" help     Displays a page of help for Teleport Ask";
  }
  
  @Override
  public List getCommandAliases() {
    return Arrays.asList(new String[] { });
  }
  
  @Override
  public void execute(MinecraftServer server, ICommandSender sender, String[] params) throws CommandException { // mc1.9
    //public void execute(                    ICommandSender sender, String[] params) throws CommandException { // mc1.8.9
    if(sender instanceof EntityPlayer) {
      EntityPlayer player = (EntityPlayer) sender;
      String playerName = player.getName(); // mc1.8
      //String playerName = player.getGameProfile().getName(); // mc1.7.10
      World world = player.worldObj;
      if ( world.isRemote ) { return; }
      MinecraftServer mcServer = ((EntityPlayerMP)player).mcServer;
      
      if ((params.length != 1) || ( params[0].equalsIgnoreCase("help"))) { ShowTIHelp(player); return; }
      
      if ( SpawnCommands.isSingleplayer ) { 
        Do.Say(player," ");
        Do.Say(player,"/\u00A7eti\u00A7r  (\u00A7eT\u00A7releport \u00A7eI\u00A7rnvite) is a Multiplayer only feature."); 
        Do.Say(player,"Sorry, there is no need for it in single player."); 
        return; 
      }
      
      // command permissions. "no" is handled by not registering the command at all. "op" and "enable" are handled here. "all" is the default.
      SpawnCommands.loadCommandPermissionsConfig();
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "ta").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"Op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "ta").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"Advanced spawn commands are not enabled."); return; }
      // the above is because this command requires the above /ta command to exist
      // the below is checking permissions for this command /ti
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "ti").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"Op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "ti").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"Advanced spawn commands are not enabled."); return; }
      
      long timeNow = (new Date()).getTime(); // in milliseconds
      String timeNowString = Long.toString(timeNow);
      
      // the invited player
      String invitedPlayerName = params[0];
      invitedPlayerName = Do.correctFromItemInArrayForCase(invitedPlayerName, mcServer.getAllUsernames());
      //EntityPlayer destinationPlayer = mcServer.getConfigurationManager().getPlayerByUsername(theName); // mc1.8 // will be null if they are not in game
      EntityPlayer invitedPlayer = Do.getPlayerByUsername(mcServer,invitedPlayerName); // mc1.8 // will be null if they are not in game
      
      if ( invitedPlayer == null ) { Do.Say(player,"\u00A7e"+params[0]+"\u00A7r is not a player in game right now."); return; }
      if ( invitedPlayerName.equalsIgnoreCase(playerName) ) { Do.Say(player,"Wow! So cool! It's like you are already here."); return; }
      
      // preload invited player's ta command with this player's name
      //ta Do.StringToFile(TAConfigBasePath+playerName+"/"+lastPlayerAskedFileName,timeNow+","+theName); // save the last /ta name for simple /ta to re issue the request
      Do.StringToFile(TAConfigBasePath+invitedPlayerName+"/"+lastPlayerAskedFileName,timeNow+","+playerName); // save the last /ti name for simple /ta to go
      
      // preapprove this request as if it were a ta request from the invited one 
      if (! Do.StringToFile(TAConfigBasePath+invitedPlayerName+"/"+incomingResponseFileName,""+timeNow+","+playerName+","+"accept") ) 
      { Do.Err(player,"Could not create file "+TAConfigBasePath+invitedPlayerName+"/"+incomingResponseFileName); return; }
      
      // notify players
      Do.Say(invitedPlayer,"\u00A7e"+playerName+"\u00A7r is asking you to teleport to them (try /ta)");
      Do.Say(player,"\u00A7e"+invitedPlayerName+"\u00A7r has been asked to teleport you."); // mc1.8
      return;

    } // end if entity player
  }
  
  public static void ShowTIHelp(EntityPlayer player) {
    Do.Say(player," ");
    SpawnCommands.loadCommandNamesConfig();
    if ( Do.getConfigArrayValue(SpawnCommands.commandNames,"ti").equalsIgnoreCase("ti") ) {
      Do.Say(player,"\u00A7eti\u00A7r means \u00A7eT\u00A7releport \u00A7eI\u00A7rnvite.");
      Do.Say(player,"Type /"+ Do.getConfigArrayValue(SpawnCommands.commandNames,"ti") +" then a space then hit tab or type the name of the player you want to invite.  The invited player should then be able to type just /"+Do.getConfigArrayValue(SpawnCommands.commandNames,"ta")+" to teleport to you.");
    }
    if (SpawnCommands.isSingleplayer) { Do.Say(player,"This is for \u00A7emultiplayer\u00A7r games only."); }
  }
  
  @Override
  public   boolean checkPermission(MinecraftServer server, ICommandSender sender) { // mc1.9
  //public boolean canCommandSenderUseCommand(             ICommandSender sender) { // mc1.8 mc1.7.10
    return true;
  }
  
  @Override
  public   List<String> getTabCompletionOptions(MinecraftServer server, ICommandSender sender, String[] args, BlockPos pos) { // mc1.9
  //public List         addTabCompletionOptions(                        ICommandSender sender, String[] args, BlockPos pos) { // mc1.8
	//public List         addTabCompletionOptions(                        ICommandSender sender, String[] args) { // mc1.7.10
    //experiment:  // how does this work in singleplayer worlds? will it crash?
    //List<String> list = Arrays.asList(((EntityPlayerMP)sender).mcServer.getConfigurationManager().getAllUsernames()); // mc1.8.9
    List<String> list = Arrays.asList(((EntityPlayerMP)sender).mcServer.getAllUsernames()); // mc1.8.9
    return list;
  }
  
  @Override
  public boolean isUsernameIndex(String[] astring, int i) {
    return false;
  }
  
}
