package aln.spawncommands;

import java.util.Arrays;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.command.CommandException;
import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Blocks;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

public class CommandUp implements ICommand {
  
  @Override
  public int compareTo(ICommand arg0) {
    return 0;
  }
  
  @Override
  public String getCommandName() {
    return Do.getConfigArrayValue(SpawnCommands.commandNames ,"up");
  }
  
  @Override
  public String getCommandUsage(ICommandSender sender) {
    return "/"+ Do.getConfigArrayValue(SpawnCommands.commandNames ,"up") +"     Teleports you up to the next highest place you can stand at this x and z coordinate.";
  }
  
  @Override
  public List<String> getCommandAliases() {
    return Arrays.asList(new String[] { });
  }
  
  @Override
  public void execute(MinecraftServer server, ICommandSender sender, String[] args) throws CommandException {
    if(sender instanceof EntityPlayer) {
      EntityPlayer player = (EntityPlayer) sender;
      World world = player.worldObj;
      if (world.isRemote) { return; }
      
      // command permissions. "no" is handled by not registering the command at all. "op" and "enable" are handled here. "all" is the default.
      SpawnCommands.loadCommandPermissionsConfig();
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "up").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"Op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "up").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"Advanced spawn commands are not enabled."); return; }
      
      // get player's location at start of this teleport request
      double  px = Math.round(player.posX - .5); // player's coordinates rounded down
      double  py = Math.round(player.posY - .5);
      double  pz = Math.round(player.posZ - .5);
      Float   pyaw = player.rotationYaw;
      Float   ppitch = player.rotationPitch;
      Integer pdim = world.provider.getDimension(); // mc1.9
      String newGoBackLocation = px +","+ py +","+ pz +","+ pyaw +","+ ppitch +","+ pdim;
      
      //ISSUE: up while in water only takes you to the surface of the water not up to solid ground; solution do nothing.
      //ISSUE: often the teleport does not trigger render of more than skyblock till you move; solution unknown.
      
      // find the next acceptable location to stand above here
      boolean foundSolidAbove = false;
      boolean foundGoodLocation = false;
      while ( (py <= world.getActualHeight() ) && ( (! foundSolidAbove) || (! foundGoodLocation) ) ) {
        if ( (! foundSolidAbove) && 
             (( ! SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) || 
              ( ! SpawnCommands.canSpawnInsideBlock(player, px, py+1, pz))
             ) 
           ) 
        {
          foundSolidAbove = true;
          py++; // up 1 because this sure isnt the spot
        }
        
        if ( ( foundSolidAbove )   && 
             ( SpawnCommands.canSpawnInsideBlock(player, px, py, pz) ) && 
             ( SpawnCommands.canSpawnInsideBlock(player, px, py+1, pz) ) 
           ) 
        {
          foundGoodLocation = true;
          break;
        }
        py++;
      }
      if ( ! foundGoodLocation ) { Do.Say(player,"You are at the highest point."); return; } else { Do.Say(player,"Up!"); }
      
      // falling check
      while (  ( py > 1 ) && ( (SpawnCommands.canSpawnInsideBlock(player, px, py - 1, pz)) && (SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) ) ) 
      { py--; }
      
      BlockPos posBelowFeet = new BlockPos(px, (py-1), pz);
      Block blockBelowFeet = player.worldObj.getBlockState(posBelowFeet).getBlock();
      
      // if at y==1 then check for no bedrock under feet 
      if ( (py == 1) && (blockBelowFeet != Blocks.BEDROCK)) { 
        player.worldObj.setBlockState(posBelowFeet, Blocks.COBBLESTONE.getDefaultState()); 
      }
      
      // lava under feet check // mc1.8,mc1.9
      if ((py > 1) && ( (blockBelowFeet == Blocks.LAVA) || (blockBelowFeet == Blocks.FLOWING_LAVA))) {
        player.worldObj.setBlockState(posBelowFeet, Blocks.COBBLESTONE.getDefaultState());
      }
      // mc1.7.10
      //if ((py>1) && ( ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.lava) || ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.flowing_lava) )) {
      //  player.worldObj.setBlock((int)px, (int)(py-1), (int)pz, Blocks.cobblestone);
      //}
      
      // actually go there now
      //((EntityPlayerMP) player).playerNetServerHandler.setPlayerLocation(px + .5d, py, pz + .5d, pyaw, ppitch);
      //player.setPositionAndUpdate(px + .5d, py, pz + .5d);
      Do.TeleportPlayerTo(player, px, py, pz);
      
      
      
      // write the location of the next go back command
      if (! Do.StringToFile( SpawnCommands.configBasePath + player.getName() + "/_" + player.getName() +"_GoBackLocation", newGoBackLocation ) ) // mc1.8 
      { Do.Err(player,"CommandSpawn: Could not create the file that stores the location to go back to."); return; }

      return;
      
    }
  }
  
  @Override
  public boolean checkPermission(MinecraftServer server, ICommandSender sender) {
    return true;
  }
  
  @Override
  public List<String> getTabCompletionOptions(MinecraftServer server, ICommandSender sender, String[] args, BlockPos pos) {
    return null;
  }
  
  @Override
  public boolean isUsernameIndex(String[] args, int index) {
    return false;
  }
  
}
