package aln.spawncommands;

import java.io.File;
import java.util.Arrays;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.command.ServerCommandManager;
import net.minecraft.entity.Entity;
import net.minecraft.entity.boss.EntityDragon;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Blocks;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.util.text.Style;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.Mod.EventHandler;
import net.minecraftforge.fml.common.Mod.Instance;
import net.minecraftforge.fml.common.SidedProxy;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPostInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.event.FMLServerStartingEvent;
import net.minecraftforge.fml.common.registry.GameData;

@Mod(modid = ModInfo.ID,name=ModInfo.NAME,version=ModInfo.VERS,acceptableRemoteVersions="*") // mc1.7.2, 1.7.10, 1.8, 1.9
//@Mod(modid=ModInfo.ID,name=ModInfo.NAME,version=ModInfo.VERS) // mc1.6.4
//@NetworkMod(clientSideRequired=false,serverSideRequired=true) // mc1.6.4

public class SpawnCommands {

  @Instance(ModInfo.ID) // are these 2 lines needed
  public static SpawnCommands instance;
  
  public static MinecraftServer mcServer = null; // needed by a function in Do.
  
  public static boolean isSingleplayer = true;
  public static boolean isMultiplayer = false; // values corrected dynamically in serverStarting()
  public static boolean isWindowsOS = System.getProperty("os.name").contains("Windows");
  public static String theNewLine = "\n"; // value corrected dynamically in serverStarting()
  public static String configBasePath = null; // always ends with a slash (/);  this is corrected dynamically in serverStarting()
  
  public static final String fileNameMiscConfig = "_miscellaneous.config";
  public static String[][] miscConfigArray = null;
  public static long miscConfigDateCode = 0; // compared to file modification date to see if reload is needed
  
  public static final String fileNameCommandPermissionsConfig = "_commandPermissions.config";
  public static String[][] commandPermissions = null;
  public static long commandPermissionsDateCode = 0; // compared to file modification date to see if reload is needed
  
  public static final String fileNameCommandNamesConfig = "_commandNames.config";
  public static String[][] commandNames = null;
  
  public static final String fileNameSpawnInBlocksConfig = "_canSpawnInsideBlockNameList.config";
  public static long spawnInBlocksDateCode = 0; // compared to file modification date to see if reload is needed
  public static String[]  spawnInBlocksID;
  //public static Integer[] spawnInBlocksMeta; // pre mc1.9 only
  public static String[]  spawnInBlocksMetaStrings;
  public static String    spawnInBlocksErrors;
  
  public static final String folderSharedNamedLocations = "_sharedNamedLocations";
  
  @SidedProxy(clientSide=ModInfo.CLIENTPROXY,serverSide=ModInfo.COMMONPROXY)
  public static ProxyCommon proxy;
  
  @EventHandler
  public void preload(FMLPreInitializationEvent event) {
    //Do.Trace("FMLPreInit finished\n");
  }
  
  @EventHandler
  public void load(FMLInitializationEvent event) {
    //Do.Trace("FMLInit finished");
  }

  @EventHandler
  public void postload(FMLPostInitializationEvent event) {
    //Do.Trace("FMLPostInit finished");
  }
  
  @EventHandler
  public void serverStarting(FMLServerStartingEvent event) {
    //System.out.println(ModInfo.NAME + " FMLServerStartingEvent started");
    mcServer=event.getServer(); // needed by a function in Do.
    if ( isWindowsOS ) { theNewLine = "\r\n"; }
    
    // detect if singleplayer or multiplayer server.  there should be a better way to detect this.
    // MinecraftServer.getServer().getConfigurationManager().getServerInstance().isSinglePlayer(); //mc1.8.9 possibly this?
    try { isSingleplayer = (Minecraft.getMinecraft() != null); } catch (NoClassDefFoundError e1) { isSingleplayer = false; }
    isMultiplayer = ! isSingleplayer;
    
    // detect what folder to use for config files. This must be before configBasePath is used.
    if ( isMultiplayer ) 
      { configBasePath = event.getServer().getEntityWorld().getSaveHandler().getWorldDirectory().getPath() +"/"+ ModInfo.ID +"-config-folder/"; } 
    else 
      { configBasePath = "./saves/"+ Minecraft.getMinecraft().getIntegratedServer().getFolderName() +"/"+ ModInfo.ID +"-config-folder/"; }
    
    // initialize config files if needed
    Do.StringToFile("./config/"+ModInfo.ID+".config","The configuration files for this mod are inside the appropriate world or /saves folder.\nLook there for the "+ModInfo.ID+" configuration files."); 
    
    // CONVERT config files from version 1.4.1 and down, to 2.0.0
    if ( Do.folderExists("./config/"+ModInfo.ID) && (! Do.folderExists(configBasePath+".")) ) { // if old config folder does exist and new one doesnt exist
      Do.moveFolder("./config/"+ModInfo.ID,configBasePath+".");
      // search for and move and rename all /back saved locations from the main config folder to the player's folder.
      File fromPathFolder = new File(configBasePath);
      String[] fileNameList = fromPathFolder.list();
      for (int i=0; i < fileNameList.length; i++) {
        if ( Do.fileExists(configBasePath+fileNameList[i]) ) {
          Pattern pattern = Pattern.compile("^(.*)_GoBackLocation$");
          Matcher matcher = pattern.matcher(fileNameList[i]);
          if ( matcher.find() ) { 
            Do.StringToFile(configBasePath+matcher.group(1)+"/_"+matcher.group(1)+"_GoBackLocation", Do.FileToString(configBasePath+fileNameList[i]) );
            File f = new File(configBasePath+fileNameList[i]);
            f.delete();
          }
        }
      } // end for
    }
    if ( Do.fileExists(configBasePath + "canSpawnInsideBlockList") ) { // rename to new
      File fh1 = new File(configBasePath + "canSpawnInsideBlockList");
      File fh2 = new File(configBasePath + fileNameSpawnInBlocksConfig);
      fh1.renameTo(fh2);
    }
    
    // make the main config folders for this mod
    Do.folderMake(configBasePath + ".");
    Do.folderMake(configBasePath + folderSharedNamedLocations);
    
    // Misc Config
    if (! Do.fileExists(configBasePath + fileNameMiscConfig)) {
      String output = 
          "AdvancedCommandsEnabled=false\nallowWithCheatsDisabled=false\nHomeWithNamedLocations=false\nAllowToLeaveTheDragon=false\nLimitTheNumberOfNamedLocationsPerPlayer=false\nTheNamedLocationsLimitNumber=3\nteleportask responseTimeLimitSeconds=45\nteleportask requestJustCameInDelaySeconds=1.2\nteleportask lastPlayerAskedTimeoutHours=9\n" +
          "away distanceAway=1500\naway distanceVariance=500\naway minDistanceFromSelf=1000\naway maxDistanceFromSelf=1000000\naway minDistanceFromWorldSpawn=500\naway coolDownMinutes=360\n"; 
      if (isWindowsOS) { output = output.replace("\n","\r\n"); }
      Do.StringToFile(configBasePath + fileNameMiscConfig,output); // defaults
    }
    loadMiscConfig();
    // CONVERT config files from version 1.4.1 and down, to 2.0.0
    if ( Do.fileExists(configBasePath + "NamesEnabled") ) {
      File fh = new File(configBasePath + "NamesEnabled");
      fh.delete();
      miscConfigArray = Do.setConfigArrayValue(SpawnCommands.miscConfigArray,"AdvancedCommandsEnabled","true");
      miscConfigArray = Do.setConfigArrayValue(SpawnCommands.miscConfigArray,"HomeWithNamedLocations","true");
      saveMiscConfig();
    }
    // CONVERT 2.0.0 to 2.1.0 (add AllowToLeaveTheDragon in The End, allows you to teleport out even if the dragon is still alive.)
    if ( Do.getConfigArrayValue(SpawnCommands.miscConfigArray, "AllowToLeaveTheDragon").equals("") ) {
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "AllowToLeaveTheDragon", "false");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "LimitTheNumberOfNamedLocationsPerPlayer", "false");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "TheNamedLocationsLimitNumber", "3");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "teleportask responseTimeLimitSeconds", "45");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "teleportask requestJustCameInDelaySeconds", "1.2");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "teleportask lastPlayerAskedTimeoutHours", "9");
      saveMiscConfig();
    }
    // CONVERT 2.1.0 to 2.7.0
    if ( Do.getConfigArrayValue(SpawnCommands.miscConfigArray,  "away distanceAway").equals("") ) {
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "away distanceAway", "1500");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "away distanceVariance", "500");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "away minDistanceFromSelf", "1000");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "away maxDistanceFromSelf", "1000000");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "away minDistanceFromWorldSpawn", "500");
      miscConfigArray = Do.setConfigArrayValue(miscConfigArray, "away coolDownMinutes", "360");
      saveMiscConfig();
    }
    
    // Command Permissions Config
    if (! Do.fileExists(configBasePath + fileNameCommandPermissionsConfig)) {
      String output = "// This file is checked each time a command in this mod is used.\n// command=[all|enabled|op|no]\n// example: back=op\n\nspawn=all\nspawnName=enabled\nspawn.=all\nspawn!=all\nspawn?=all\nback=enabled\nspawn+=all\nspawn+enable=op\nspawn+@sharedlocation=op\nspawn+name=enabled\nspawn+nameWithCoordinates=op\nspawn-=all\nspawn-name=enabled\nspawn-@sharedlocation=op\nspawns=all\nhome=all\nsethome=all\nspawnhelp=all\nspawnblockdata=op\nta=enabled\nti=enabled\nok=enabled\ntop=op\nup=op\ndn=op\naway=enabled\n";
      if (SpawnCommands.isWindowsOS) { output.replace("\n","\r\n"); }
      Do.StringToFile( configBasePath + fileNameCommandPermissionsConfig, output ); // defaults
    }
    loadCommandPermissionsConfig();
    // CONVERT 2.0.0 to 2.1.0 (add new commands)
    if ( Do.getConfigArrayValue(commandPermissions, "spawnblockdata").equals("") ) {
      commandPermissions = Do.setConfigArrayValue(SpawnCommands.commandPermissions, "spawnblockdata", "op");
      commandPermissions = Do.setConfigArrayValue(SpawnCommands.commandPermissions, "ta", "enabled");
    }
    // CONVERT 2.1.0 to 2.7.0 (add new commands)
    if ( Do.getConfigArrayValue(commandPermissions, "ti").equals("") ) {
      commandPermissions = Do.setConfigArrayValue(SpawnCommands.commandPermissions, "ti", "enabled");
      commandPermissions = Do.setConfigArrayValue(SpawnCommands.commandPermissions, "ok", "enabled");
      commandPermissions = Do.setConfigArrayValue(SpawnCommands.commandPermissions, "top", "op");
      commandPermissions = Do.setConfigArrayValue(SpawnCommands.commandPermissions, "up", "op");
      commandPermissions = Do.setConfigArrayValue(SpawnCommands.commandPermissions, "dn", "op");
      commandPermissions = Do.setConfigArrayValue(SpawnCommands.commandPermissions, "away", "enabled");
    }
    saveCommandPermissionsConfig();
    
    // Command Names Config
    if (! Do.fileExists(configBasePath + fileNameCommandNamesConfig)) {
      String output = "// This file is loaded once on server startup\n// Deleting this file will reset it to defaults on game restart\n// originalCommand=newCommand\n// example: back=goback\nOldCommandName=NewCommandNameExample\nspawn=spawn\nspawn.=spawn.\nspawn!=spawn!\nspawn?=spawn?\nback=back\nspawn+=spawn+\nspawn-=spawn-\nspawns=spawns\nhome=home\nsethome=sethome\nspawnhelp=spawnhelp\nspawnblockdata=spawnblockdata\nta=ta\nti=ti\nok=ok\ntop=top\nup=up\ndn=dn\naway=away\n";
      if (SpawnCommands.isWindowsOS) { output.replace("\n","\r\n"); }
      Do.StringToFile(configBasePath + fileNameCommandNamesConfig, output ); // defaults
    }
    loadCommandNamesConfig();
    // CONVERT 2.0.0 to 2.1.0 (add new commands)
    if ( Do.getConfigArrayValue(commandNames, "spawnblockdata").equals("") ) {
      commandNames = Do.setConfigArrayValue(SpawnCommands.commandNames, "spawnblockdata", "spawnblockdata");
      commandNames = Do.setConfigArrayValue(SpawnCommands.commandNames, "ta", "ta");
    }
    // CONVERT 2.1.0 to 2.7.0 (add new commands)
    if ( Do.getConfigArrayValue(commandNames, "ti").equals("") ) {
      commandNames = Do.setConfigArrayValue(SpawnCommands.commandNames, "ti", "ti");
      commandNames = Do.setConfigArrayValue(SpawnCommands.commandNames, "ok", "ok");
      commandNames = Do.setConfigArrayValue(SpawnCommands.commandNames, "top", "top");
      commandNames = Do.setConfigArrayValue(SpawnCommands.commandNames, "up", "up");
      commandNames = Do.setConfigArrayValue(SpawnCommands.commandNames, "dn", "dn");
      commandNames = Do.setConfigArrayValue(SpawnCommands.commandNames, "away", "away");
    }
    saveCommandNamesConfig();
    
    // Spawn In Blocks Config        must be done postload or after to include the blocks from all mods.
    if ( ! Do.fileExists(configBasePath + fileNameSpawnInBlocksConfig)) { // for mc 1.7.2+ NameList
      String output =       
          "// Block names you add here allow the player to spawn into them. Some vanilla blocks are hard coded:\n" +
          "// air, bed, tallgrass, red_flower, yellow_flower, deadbush, vine, snow_layer, and torch.\n" +
          "// Adding a comma then the [additional data] to an entry specifies the metadata to match.\n" + 
          "// If there is just a block name then additional data is ignored.\n"+ 
          "// Example:  minecraft:sapling,[stage=0,type=birch]  would compare block 'minecraft:sapling' to the block id in question and compare its additional data to [stage=0,type=birch].\n" +
          "// This would allow you to spawn while standing inside a Birch tree sapling specifically.\n" +
          "// While in game type the command /"+ Do.getConfigArrayValue(commandNames,"spawnblockdata")+"\n" +
          "// One entry per line and as many lines as you want.  Comment lines may be removed if you wish.\n" +
          "\n\nThaumcraft:blockAiry\nRailcraft:residual.heat\nRailcraft:tile.railcraft.residual.heat\nTConstruct:decoration.stonetorch\n" +
          "\n\n// Generated list of block names: (delete this file and restart the server to refresh this list) ";
      for ( Block blockModAndName : GameData.getBlockItemMap().keySet() ) { output += "  " + blockModAndName.getRegistryName().toString(); } // mc1.9
      //for ( Object i : GameData.getBlockRegistry().getKeys() ) { GeneratedNameList += "  " + i.toString(); } // 1.8.9, depreciated 1.9+
      output += "\n";
      if (isWindowsOS) { output = output.replace("\n","\r\n"); }
      Do.StringToFile(configBasePath + fileNameSpawnInBlocksConfig,output);
    }
    
    // Forge Hooks       (death event etc)
    MinecraftForge.EVENT_BUS.register(new ForgeHooks()); // must be after the configBase has been defined.
    
    //event.registerServerCommand(new CommandTest()); // mc1.8
    ServerCommandManager manager = (ServerCommandManager) mcServer.getCommandManager(); // mc1.9
    
    // register commands, if they are not disabled in the config
    //ServerCommandManager manager = (ServerCommandManager) MinecraftServer.getServer().getCommandManager(); // mc1.8.9
    //{ manager.registerCommand(new CommandSpawn()); } // mc1.8.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"spawn").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSpawn()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"spawn.").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSpawnPoint()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"spawn!").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSpawnSet()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"spawn?").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSpawnQuery()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"spawn+").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSpawnPlus()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"spawn-").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSpawnMinus()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"spawns").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSpawns()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"home").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandHome()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"sethome").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSetHome()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"spawnhelp").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSpawnHelp()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"back").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandBack()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"spawnblockdata").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandSpawnBlockData()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"ta").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandTeleportAsk()); } // mc1.9
    // version 2.7.0
    if ( ! Do.getConfigArrayValue(commandPermissions,"ti").equalsIgnoreCase("no") ) // telelport invite
    { manager.registerCommand(new CommandTeleportInvite()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"ok").equalsIgnoreCase("no") ) // /ta ok
    { manager.registerCommand(new CommandOK()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"top").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandTop()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"up").equalsIgnoreCase("no") ) 
    { manager.registerCommand(new CommandUp()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"dn").equalsIgnoreCase("no") ) // down
    { manager.registerCommand(new CommandDown()); } // mc1.9
    if ( ! Do.getConfigArrayValue(commandPermissions,"away").equalsIgnoreCase("no") ) // far away randomly
    { manager.registerCommand(new CommandAway()); } // mc1.9
    
    System.out.println(ModInfo.NAME +" v"+ ModInfo.VERS +" ready");
  } // end serverLoad
  
  
  
  
  
  public static void showHelp(EntityPlayer player) {
    Do.Say(player," ");
    Do.Say(player,"\u00A7R"+ ModInfo.NAME +" v"+ ModInfo.VERS +" quick help:");
    Do.Say(player," /"+ Do.getConfigArrayValue(commandNames,"spawn")   +"    To the main world spawn point.");
    Do.Say(player," /"+ Do.getConfigArrayValue(commandNames,"spawn.")  +"    To your bed/home location if it is set.");
    Do.Say(player," /"+ Do.getConfigArrayValue(commandNames,"spawn!")  +"    Sets your bed/home location.");
    Do.Say(player," /"+ Do.getConfigArrayValue(commandNames,"spawn?")  +"    Displays coordinates.");
    Do.Say(player," \u00A7e/"+Do.getConfigArrayValue(commandNames,"spawn+") +" mylavabase\u00A7R  Creates your named spawn point.");
    Do.Say(player," \u00A7e/"+Do.getConfigArrayValue(commandNames,"spawn")  +" mylavabase\u00A7R     Teleports you there.");
    Do.Say(player," /"+ Do.getConfigArrayValue(commandNames,"spawn-")  +" mybasename     Removes a named location.");
    
    Do.Say(player," /"+ Do.getConfigArrayValue(commandNames,"spawn+")  
        +" enable|disable  /"+ Do.getConfigArrayValue(commandNames,"spawns")
        +" /"+ Do.getConfigArrayValue(commandNames,"top")
        +" /"+ Do.getConfigArrayValue(commandNames,"up")
        +" /"+ Do.getConfigArrayValue(commandNames,"dn")
        +" /"+ Do.getConfigArrayValue(commandNames,"spawn") 
        +" help");
    Do.Say(player," /"+ Do.getConfigArrayValue(commandNames,"home")    
        +" /"+ Do.getConfigArrayValue(commandNames,"sethome") 
        +" /"+ Do.getConfigArrayValue(commandNames,"back") 
        +" /"+ Do.getConfigArrayValue(commandNames,"spawn+") 
        +" myplace -123 65 42 /"+ Do.getConfigArrayValue(commandNames,"away") +" \u00A7e/" + Do.getConfigArrayValue(commandNames,"ta")+"\u00A7r");
    if ( ( ! Do.getConfigArrayValue(commandPermissions,"spawnblockdata").equalsIgnoreCase("no") ) && ( Do.IsOp(player) ) ) 
    { Do.Say(player," /"+ Do.getConfigArrayValue(commandNames,"spawnblockdata")+"  Displays block data where standing."); }
    
    return;
  }
  
  public static void loadMiscConfig() {
    if ( miscConfigDateCode != Do.fileDateModified(configBasePath + fileNameMiscConfig) ) { // only load if changed.
     	 miscConfigDateCode    = Do.fileDateModified(configBasePath + fileNameMiscConfig); // new reference point for the date modified
     	 miscConfigArray     = Do.getConfigArrayFile(configBasePath + fileNameMiscConfig);
    }
  }
  public static boolean saveMiscConfig() {
    return Do.putConfigArrayFile(configBasePath + fileNameMiscConfig, miscConfigArray);
  }
	
  public static void loadCommandPermissionsConfig() {
    if ( commandPermissionsDateCode != Do.fileDateModified(configBasePath + fileNameCommandPermissionsConfig) ) { // only load if changed.
    	 commandPermissionsDateCode    = Do.fileDateModified(configBasePath + fileNameCommandPermissionsConfig); // new reference point for the date modified
    	 commandPermissions          = Do.getConfigArrayFile(configBasePath + fileNameCommandPermissionsConfig);
    }
  }
  public static boolean saveCommandPermissionsConfig() {
    return Do.putConfigArrayFile(configBasePath + fileNameCommandPermissionsConfig, commandPermissions);
  }
  
  public static void loadCommandNamesConfig() {
    commandNames = Do.getConfigArrayFile(configBasePath + fileNameCommandNamesConfig);
  }
  public static boolean saveCommandNamesConfig() {
    return Do.putConfigArrayFile(configBasePath + fileNameCommandNamesConfig, commandNames);
  }
  
  public static void listNamedLocations(EntityPlayer player) {
    // get, sort, and display the list of SHARED named locations
    File thefolder = new File(configBasePath+folderSharedNamedLocations );
    String[] filesInDir = thefolder.list();
    Arrays.sort(filesInDir);
    int locationCount = 0;
    
    SpawnCommands.loadCommandPermissionsConfig();
    
    ITextComponent tcomp = null;
    TextComponentString theListTCS = new TextComponentString("");
    
    //player.addChatComponentMessage(new ChatComponentText(" ")); // mc1.8.9
    player.addChatComponentMessage(new TextComponentString(" ")); // mc1.9
    
    // display before the list of locations.  Only display these commands if they are available to the player.
    tcomp = new TextComponentString("clickable ");
    theListTCS.appendSibling(tcomp);
    
    if (   (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawnhelp").equalsIgnoreCase("op") && (! Do.IsOp(player)) ))
        && (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawnhelp").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ))
        && (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawnhelp").equalsIgnoreCase("no") ))
       )
    {
      tcomp = new TextComponentString("?");
      //comp.setChatStyle(new ChatStyle().setChatClickEvent(new ClickEvent(Action.RUN_COMMAND, "/"+Do.getConfigArrayValue(commandNames,"spawnhelp") ) {})); // mc1.8.9 down
      //tcomp.setStyle(new Style().setClickEvent(new net.minecraft.util.text.event.ClickEvent(net.minecraft.util.text.event.ClickEvent.Action.RUN_COMMAND, "/"+Do.getConfigArrayValue(commandNames,"spawnhelp")  ) {})); // mc1.9
      tcomp.setStyle(  new Style().setClickEvent(new net.minecraft.util.text.event.ClickEvent(net.minecraft.util.text.event.ClickEvent.Action.RUN_COMMAND, "/"+Do.getConfigArrayValue(commandNames,"spawnhelp")  )   )); // mc1.9.4
      theListTCS.appendSibling(tcomp);
    }
    
    if (   (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn").equalsIgnoreCase("op") && (! Do.IsOp(player)) ))
        && (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ))
        && (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn").equalsIgnoreCase("no") ))
       )
    {
      tcomp = new TextComponentString(" ");
      theListTCS.appendSibling(tcomp);
      tcomp = new TextComponentString("/"+Do.getConfigArrayValue(commandNames,"spawn"));
      tcomp.setStyle(new Style().setClickEvent(new net.minecraft.util.text.event.ClickEvent(net.minecraft.util.text.event.ClickEvent.Action.RUN_COMMAND, "/"+Do.getConfigArrayValue(commandNames,"spawn") )   ));
      theListTCS.appendSibling(tcomp);
    }
    
    if (   (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn.").equalsIgnoreCase("op") && (! Do.IsOp(player)) ))
        && (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn.").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ))
        && (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn.").equalsIgnoreCase("no") ))
       )
    {
      tcomp = new TextComponentString(" ");
      theListTCS.appendSibling(tcomp);
      tcomp = new TextComponentString("/"+Do.getConfigArrayValue(commandNames,"spawn."));
      tcomp.setStyle(new Style().setClickEvent(new net.minecraft.util.text.event.ClickEvent(net.minecraft.util.text.event.ClickEvent.Action.RUN_COMMAND, "/"+Do.getConfigArrayValue(commandNames,"spawn.") )   ));
      theListTCS.appendSibling(tcomp);
    }
    
    if (   (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "back").equalsIgnoreCase("op") && (! Do.IsOp(player)) ))
        && (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "back").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ))
        && (!( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "back").equalsIgnoreCase("no") ))
       )
    {
      tcomp = new TextComponentString(" ");
      theListTCS.appendSibling(tcomp);
      tcomp = new TextComponentString("/"+Do.getConfigArrayValue(commandNames,"back"));
      tcomp.setStyle(new Style().setClickEvent(new net.minecraft.util.text.event.ClickEvent(net.minecraft.util.text.event.ClickEvent.Action.RUN_COMMAND, "/"+Do.getConfigArrayValue(commandNames,"back") )   ));
      theListTCS.appendSibling(tcomp);
    }
    
    player.addChatComponentMessage(theListTCS);
    theListTCS = new TextComponentString("");
    
    tcomp = new TextComponentString("  \u00A7e:");
    theListTCS.appendSibling(tcomp);
    // "\u00A7e" text chat color code e;
    
    // loop though all the SHARED named locations
    for ( int i=0; i<filesInDir.length; i++ ) {
      // get named location data
      String fc = Do.FileToString(configBasePath+folderSharedNamedLocations+"/"+filesInDir[i]); // file content
      String[] fcValues = fc.split(","); // parsed
      if ( fcValues.length != 6 ) { Do.Err("Internal Error: expected 5 values: x y z yaw pitch then dimensionID. Location: "+filesInDir[i] +" in the shared locations folder."); }
      String theColor = "";
      if ( Integer.parseInt(fcValues[5]) != player.dimension ) { theColor="\u00A77"; } else { theColor="\u00A7e"; } // mc1.9
      //if ( Integer.parseInt(fcValues[5]) != player.worldObj.provider.getDimensionId() ) { theColor="\u00A77"; } else { theColor="\u00A7e"; } // mc1.8
      //if ( Integer.parseInt(fcValues[5]) != player.worldObj.provider.dimensionId ) { theColor="\u00A77"; } else { theColor="\u00A7e"; } // mc1.7.10
      tcomp = new TextComponentString("  ");
      theListTCS.appendSibling(tcomp);
      tcomp = new TextComponentString("\u00A7a@\u00A7r"+theColor+filesInDir[i]+"\u00A7r");
      tcomp.setStyle(new Style().setClickEvent(new net.minecraft.util.text.event.ClickEvent(net.minecraft.util.text.event.ClickEvent.Action.RUN_COMMAND, "/"+Do.getConfigArrayValue(commandNames,"spawn")+" @"+filesInDir[i])   ));
      theListTCS.appendSibling(tcomp);
      locationCount++;
    }
    
    // get, sort, and display the list of named locations for this PLAYER
    Do.folderMake(configBasePath + player.getName()); // mc1.8
    thefolder = new File(configBasePath + player.getName()); // mc1.8
    filesInDir = thefolder.list();
    if ( ((filesInDir == null) || ( filesInDir.length == 0 )) && (locationCount == 0)) { Do.Say(player, "You have no named locations."); return; }
    Arrays.sort(filesInDir);
    for ( int i=0; i<filesInDir.length; i++ ) {
      if ( ! filesInDir[i].substring(0,1).equals("_") ) { // if not the go back file or other system file 
        // get named location data
        String fc = Do.FileToString(configBasePath+player.getName()+"/"+filesInDir[i]); // mc1.8 // file content
        String[] fcValues = fc.split(","); // parsed
        if ( fcValues.length != 6 ) { Do.Err("Internal Error: expected 5 values: x y z yaw pitch then dimensionID. Location: "+filesInDir[i] +" for player "+ player.getName() ); } // mc1.8 
        String theColor = "";
        if ( Integer.parseInt(fcValues[5]) != player.dimension ) { theColor="\u00A77"; } else { theColor="\u00A7e"; } // mc1.9
        //if ( Integer.parseInt(fcValues[5]) != player.DimensionId() ) { theColor="\u00A77"; } else { theColor="\u00A7e"; } // mc1.8 
        //if ( Integer.parseInt(fcValues[5]) != player.worldObj.provider.dimensionId ) { theColor="\u00A77"; } else { theColor="\u00A7e"; } // mc1.7.10 
        tcomp = new TextComponentString("  ");
        theListTCS.appendSibling(tcomp);
        tcomp = new TextComponentString(theColor+filesInDir[i]+"\u00A7r");
        tcomp.setStyle(new Style().setClickEvent(new net.minecraft.util.text.event.ClickEvent(net.minecraft.util.text.event.ClickEvent.Action.RUN_COMMAND, "/"+Do.getConfigArrayValue(commandNames,"spawn")+" "+filesInDir[i]) ));
        theListTCS.appendSibling(tcomp);
        locationCount++;
      }
    }
    if ( locationCount == 0 ) { Do.Say(player, "You have no named locations."); return; }
    tcomp = new TextComponentString("  \u00A7e.\u00A7r");
    theListTCS.appendSibling(tcomp);
    player.addChatComponentMessage(theListTCS); // is there a length limit to chat messages?
    return;
  }
  
  public static boolean advancedCommandsAreEnabled() {
    loadMiscConfig();
    return Do.getConfigArrayValueBoolean(miscConfigArray,"AdvancedCommandsEnabled");
  }
  
  public static boolean canSpawnInsideBlock(EntityPlayer player, BlockPos pos) {
    return canSpawnInsideBlock(player,pos.getX(),pos.getY(),pos.getZ());
  }
  public static boolean canSpawnInsideBlock(EntityPlayer player, double x, double y, double z) {
    return canSpawnInsideBlock(player,(int)x,(int)y,(int)z);
  }
  public static boolean canSpawnInsideBlock(EntityPlayer player, int x, int y, int z) {
    World world = player.worldObj;
    String[] spawnInBlocksLines = null;
    if ( y > world.getActualHeight() ) { return true; }
    if ( y < 1 ) { return false; }
    
    BlockPos pos = new BlockPos(x, y, z); // mc1.8, mc1.9
    IBlockState iBlockState= world.getBlockState(pos);
    Block blockID = iBlockState.getBlock(); // mc1.8, mc1.9
    //Block blockID = player.worldObj.getBlock(x, y, z); // mc1.7.10
    
    //interesting info:
    //  http://mcforge.readthedocs.io/en/latest/blockstates/states/  
    //  http://minecraft.gamepedia.com/Minecraft_Wiki
    
    String blockMetaDataString = Do.blockMetadataString(player, pos); // mc1.9
    //int blockMetaData = player.worldObj.getChunkFromBlockCoords(pos).getBlockMetadata(pos); // mc1.8
    //int blockMetaData = player.worldObj.getBlockMetadata(x,y,z); // mc1.7.10
    
    // if standard IDs then its ok
    if 
    ( 
      (blockID.isPassable(world, pos) && (! blockID.getMaterial(iBlockState).isLiquid()) ) ||
      (blockID == Blocks.BED) // yes it is ok to spawn inside a bed.
      // add hard coded exceptions here
      // this does seem to allow spawning inside plants
    )
    { return true; }
    // read config file for spawning inside blocks
    String fileName = configBasePath + fileNameSpawnInBlocksConfig;
    if ( spawnInBlocksDateCode != Do.fileDateModified(fileName) ) { // only load if changed.
         spawnInBlocksDateCode  = Do.fileDateModified(fileName); // new reference point for the date modified
      spawnInBlocksErrors = "";
      String fileContent = Do.FileToString(fileName); // file content 
      fileContent = fileContent.replaceAll("\r","\n"); // fix windows style text files
      spawnInBlocksLines = fileContent.split("\n"); // split from string to string[]
      spawnInBlocksID = new String[spawnInBlocksLines.length];
      //spawnInBlocksMeta = new Integer[spawnInBlocksLines.length];
      spawnInBlocksMetaStrings = new String[spawnInBlocksLines.length];
      for (int i = 0; i < spawnInBlocksLines.length; i++ ) {
        spawnInBlocksLines[i] = spawnInBlocksLines[i].replaceAll("^\\s+",""); // remove leading white space
        spawnInBlocksID[i] = "";
        //spawnInBlocksMeta[i] = -1;
        spawnInBlocksMetaStrings[i] = null;
        // if the line has something in it and its not starting with a comment then...
        if ( (spawnInBlocksLines[i] != null) && ( spawnInBlocksLines[i].length() != 0 ) && ( ! spawnInBlocksLines[i].startsWith("//")) ) {
          String line = spawnInBlocksLines[i].replaceAll("//.*$",""); // remove all comments
          line = line.replaceAll("^[\\s\\,:]+",""); // remove leading white space, commas, and colons
          line = line.replaceAll("[ ]+",","); // compatible with space or comma separators
          line = line.replaceAll("[\\s\\n\\r]$",""); // remove all trailing white space and newlines
          String[] lineContent = line.split(",",2); // line content
          // error check and assign into arrays
          if ( lineContent.length >= 1 ) { // at least 1 parameter blockID

        	  // example:   Railcraft:residual.heat
            spawnInBlocksID[i] = lineContent[0];
            
            // is this block id in the block registry?
            // if ( Block.getBlockFromName(lineContent[0]) == null ) { // probably mc1.7.10
            // if ( GameData.getBlockRegistry().getObject(lineContent[0]) == null ) { // mc1.8
            ResourceLocation itemName = new ResourceLocation(lineContent[0]); // mc1.8.9
            //if ( GameData.getBlockRegistry().getId(itemName) == -1 ) { zzzz;// mc1.8.9
            if ( Do.blockModAndName(player, x, y, z) == null ) { // mc1.9
              spawnInBlocksErrors += "SpawnCommands Config: file "+fileNameSpawnInBlocksConfig+" line "+(i+1)+" \u00A7eblock name\u00A7R '"+lineContent[0]+"' does not exist.  "; 
              spawnInBlocksID[i] = ""; 
            }
            if ( lineContent.length >= 2 ) { // 2 parameters: ID and meta data
              /**
              try { spawnInBlocksMeta[i] = Integer.parseInt(lineContent[1]); } catch (NumberFormatException e) { 
                spawnInBlocksErrors += "SpawnCommands Config: file "+fileNameSpawnInBlocksConfig+" line "+(i+1)+" \u00A7eMetadata\u00A7R '"+lineContent[1]+"' is not a number.  "; 
                spawnInBlocksMeta[i] = -1;
              }
              /**/
              spawnInBlocksMetaStrings[i] = lineContent[1];
            }
          } // end len >=1
        }
      } // end for i 
    } // end if modified then load data from file into spawnInBlocksID and spawnInBlocksMeta    
    // loop through and compare IDs and Metas
    if ( spawnInBlocksErrors != "") { Do.Err(spawnInBlocksErrors); }
    
    String blockModNameAndName = blockID.getRegistryName().toString();
    //String blockIDName = blockID.getRegistryName(); // mc1.8.9  -- example data: "minecraft:stone"
    for (int i = 0; i < spawnInBlocksID.length; i++ ) {
      if (spawnInBlocksID[i].length() > 0) { // dont bother checking if the line is blank
        //if ( spawnInBlocksMeta[i] == -1 ) { // pre mc1.9
        if ( spawnInBlocksMetaStrings[i] == null ) {
          //if (                        Block.getBlockFromName(spawnInBlocksID[i]) == blockID ) { return true; } // probably mc1.7.10
          //if (  ((Block)GameData.getBlockRegistry().getObject(spawnInBlocksID[i])) == blockID ) { return true; } // mc1.8
          if   ( spawnInBlocksID[i].equalsIgnoreCase(blockModNameAndName) ) { return true; } // mc1.8.9, mc1.9
        } else {
          //if ((                         Block.getBlockFromName(spawnInBlocksID[i]) == blockID) && ( spawnInBlocksMeta[i] == blockMetaData )) { return true; }
          //if (( ((Block)GameData.getBlockRegistry().getObject(spawnInBlocksID[i])) == blockID) && ( spawnInBlocksMeta[i] == blockMetaData )) { return true; } // mc1.8
          //if ( ( spawnInBlocksID[i].equals(blockIDName) ) && ( spawnInBlocksMetaStrings[i] == blockMetaData ) ) { return true; } // mc1.8.9
          if (  ( spawnInBlocksID[i].equalsIgnoreCase(blockModNameAndName) ) 
             && ( spawnInBlocksMetaStrings[i].equalsIgnoreCase(blockMetaDataString) ) 
             ) { return true; } // mc1.8.9, mc1.9
        }
      }
    } // end for i 
    return false;
  }
  
  public static void SetBedSpawnPoint(EntityPlayer player) {
    int playerDim = player.dimension; // mc1.9
    //int playerDim = player.worldObj.provider.getDimensionId(); // mc1.8
    //int playerDim = player.worldObj.provider.dimensionId; // mc1.7.10
    
    if ( !(player.worldObj.provider.isSurfaceWorld()) ) {  // rule from EntityPlayer.trySleep(BlockPos)
      Do.Say(player, "Cannot set the bed/home location in this dimension.");
      return;
    }
    
    // get bed current location
    BlockPos bedCoordinates = player.getBedLocation(playerDim); // mc1.8
    //ChunkCoordinates bedCoordinates = player.getBedLocation(playerDim); // mc1.7.10
    //if ( bedCoordinates == null) { bedCoordinates = new ChunkCoordinates(); } // mc1.7.10

    // get players current location
    double px = Math.round(player.posX - .5); // player's coordinates rounded down
    double py = Math.round(player.posY - .5);
    double pz = Math.round(player.posZ - .5); // bed location uses no yaw and pitch

    // Don't spawn inside blocks check
    while ( ( py < player.worldObj.getActualHeight() ) 
        &&  ( (!SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) || (!SpawnCommands.canSpawnInsideBlock(player, px, py + 1, pz)) )
          ) 
    { py++; }
    
    // falling check
    while ( ( py > 1 ) 
        &&  ( (SpawnCommands.canSpawnInsideBlock(player, px, py - 1, pz)) && (SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) )
          ) 
    { py--; }
    
    // set the spawn point for their bed/home
    player.setSpawnChunk(new BlockPos(px,py,pz), true, playerDim); // mc1.8
    //bedCoordinates.set((int)px, (int)py, (int)pz); // mc1.7.10
    
    // verify it worked in this dimension
    BlockPos realBedPos = player.getBedLocation(playerDim); // mc1.8
    //ChunkCoordinates realBedCC = player.getBedLocation(playerDim); // mc1.7.10
    if ( ( realBedPos == null ) || (realBedPos.getX() != px) || (realBedPos.getY() != py) || (realBedPos.getZ() != pz) ) { // mc1.8
    //if ( ( realBedCC == null ) || (realBedCC.posX != px) || (realBedCC.posY != py) || (realBedCC.posZ != pz) ) { // mc1.7.10
      Do.Say(player, "\u00A74Failed\u00A7r to set bed/home location"+ (playerDim != 0 ? " in this dimension" : "") +"." );
      return;
    }
    
    // lava under feet check // mc1.8
    BlockPos posBelowFeet = new BlockPos(px, (py-1), pz);
    if ((py>1) && ( ( player.worldObj.getBlockState(posBelowFeet).getBlock() == Blocks.LAVA) || ( player.worldObj.getBlockState(posBelowFeet).getBlock() == Blocks.FLOWING_LAVA) )) {
      player.worldObj.setBlockState(posBelowFeet, Blocks.COBBLESTONE.getDefaultState());
    }
    // lava under feet check // mc1.7.10
    //if ((py>1) && ( ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.lava) || ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.flowing_lava) )) {
    //  player.worldObj.setBlock((int)px, (int)(py-1), (int)pz, Blocks.cobblestone);
    //}
    
    // actually go there
    //player.setPositionAndUpdate(px + .5, py, pz + .5); // doesn't use yaw and pitch for bed
    Do.TeleportPlayerTo(player, px, py, pz);
    
    Do.Say(player, "Your bed/home location is now set.");
    return;
  }
  
  public static void GoToBedSpawnPoint(EntityPlayer player) {
    World world = player.worldObj;
    // get player's location at start of this teleport request
    double  px = Math.round(player.posX - .5); // player's coordinates rounded down
    double  py = Math.round(player.posY - .5);
    double  pz = Math.round(player.posZ - .5);
    Float   pyaw = player.rotationYaw;
    Float   ppitch = player.rotationPitch;
    int     originDim = world.provider.getDimension(); // mc1.9
    //int     originDim = world.provider.getDimensionId(); // mc1.8
    //int     originDim = world.provider.dimensionId; // mc1.7.10
    String newGoBackLocation = px +","+ py +","+ pz +","+ pyaw +","+ ppitch +","+ originDim;
    
    // get bed location
    int bedDim;
    boolean bedIsInThisDim;
    
    /**
    // dumps list of all loaded mod IDs to server console. un-comment section to use.
    List<ModContainer> theMods = Loader.instance().getModList();
    System.out.println("MOD LIST DUMP start");
    System.out.print("\n\n");
    Iterator<ModContainer> it1 = theMods.iterator(); 
    while (it1.hasNext()) { System.out.print(it1.next().getModId()+" "); }
    System.out.print("\n\n\n");
    System.out.println("MOD LIST DUMP stop");
    /**/

    // mc1.8
    BlockPos bedPos = player.getBedLocation(originDim); // mc1.8
    //ChunkCoordinates bedCC = player.getBedLocation(originDim); // mc1.7.10
    if ( bedPos == null ) { bedIsInThisDim = false; } else { bedIsInThisDim = true; } // mc1.8, mc1.9
    //if ( bedCC == null ) { bedIsInThisDim = false; } else { bedIsInThisDim = true; } // mc1.7.10
    if ( bedIsInThisDim ) { bedDim = originDim; } else { bedDim = 0; } // mc1.8, mc1.9
    //if ( bedIsInThisDim ) { bedDim = originDim; } else { bedDim = 0; } // mc1.7.10
    if (  Do.isModIDLoaded("TwilightForest") 
       && (world.provider.getDimensionType().getName().equals("Twilight Forest")) // mc1.9
       //&& (world.provider.getDimensionName().equals("Twilight Forest")) // mc1.8 
       && bedIsInThisDim 
       ) 
    { bedDim = 0; bedIsInThisDim = false; } // exception for TwilightForest mod
    
    bedPos = player.getBedLocation(bedDim); // mc1.8, mc1.9
    //bedCC = player.getBedLocation(bedDim); // mc1.7.10
    if ( bedPos == null) { Do.Say(player, "Bed spawn point is not set."); return; } // mc1.8
    //if ( bedCC == null) { Do.Say(player, "Bed spawn point is not set."); return; } // mc1.7.10
    
    // if changing dimensions
    if (! bedIsInThisDim ) { 
      // check if in The End dimension and if the dragon is alive
      boolean dragonIsAlive = false;
      if ( originDim == 1 ) { // if in The End ...
        for ( int k = 0; k < world.loadedEntityList.size(); k++ ) {
          Entity it = (Entity) world.loadedEntityList.get(k);
          if ( it instanceof EntityDragon ) { 
            dragonIsAlive = true;
            k = world.loadedEntityList.size();
          }
        }
      } // end dragon check
      // if trying to go out of The End
      SpawnCommands.loadMiscConfig();
      if (( originDim == 1 ) && ( bedDim != 1 ) && ( dragonIsAlive ) && (! Do.getConfigArrayValueBoolean(SpawnCommands.miscConfigArray, "AllowToLeaveTheDragon")) ) 
      { Do.Say(player,"The mighty dragon is in control of this world. You must defeat the dragon or death is your only escape!"); return; }
      // The following is a hack fix that overcomes a problem when directly leaving the end to any other dimension.
      // The problem: when you leave the end to another dimension the world will NOT load.
      // The solution: when you go to another dimension then go to a third one it WILL load, so we go to another one on the way.
      if ( ( originDim == 1 ) && ( bedDim != 1 ) ) {
        if ( bedDim == 0 ) { player.changeDimension(-1); } else { player.changeDimension(0); } // mc1.9
        //if ( bedDim == 0 ) { player.travelToDimension(-1); } else { player.travelToDimension(0); } // mc1.8.9 
      }
      if (player.dimension != bedDim) {
        player.changeDimension(bedDim); // officially change dimension now // mc1.9
      }
      //player.travelToDimension(bedDim); // officially change dimension now // mc1.8.9
    } // end if changing dimensions
    
    // mc1.8
    px = bedPos.getX();
    py = bedPos.getY();
    pz = bedPos.getZ(); // bed locations don't need yaw, or pitch
    // mc1.7.10
    //px = bedCC.posX;
    //py = bedCC.posY;
    //pz = bedCC.posZ; // bed locations don't need yaw, or pitch
    
    // write the location of the next go back command
    if (! Do.StringToFile(configBasePath+player.getName()+"/_"+player.getName()+"_GoBackLocation", newGoBackLocation ) ) // mc1.8 
    { Do.Err(player,"SpawnCommands: GoToBedSpawnPoint: Could not create the file that stores the location to go back to."); return; }
    
    boolean spawnWasBlocked = false;
    
    // check if destination is an actual bed block, if so then try to move the bed location to beside it or last resort just above it.
    // this is to possibly help the problem of spawning inside the stone below the bed on slow load times.
    Block blockID = world.getBlockState(new BlockPos(px, py, pz)).getBlock(); // mc1.8
    //Block blockID = player.worldObj.getBlock((int)px, (int)py, (int)pz); // mc1.7.10
    if ( blockID == Blocks.BED ) { 
      // check around the bed in different directions
      Integer[][] searchGrid = 
      { 
        {-1, 0,0},{1, 0,0},{0, 0,-1},{0, 0,1},{-1, 0,-1},{1, 0,-1},{-1, 0,1},{1, 0,1},   
        {-1, 1,0},{1, 1,0},{0, 1,-1},{0, 1,1},{-1, 1,-1},{1, 1,-1},{-1, 1,1},{1, 1,1},   
        {-1,-1,0},{1,-1,0},{0,-1,-1},{0,-1,1},{-1,-1,-1},{1,-1,-1},{-1,-1,1},{1,-1,1},   
        {0,1,0} 
      }; // x y z offsets
      for (int i = 0;i<searchGrid.length;i++) {
        blockID=world.getBlockState(new BlockPos(px+searchGrid[i][0],      py+searchGrid[i][1]  ,      pz+searchGrid[i][2])).getBlock(); // mc1.8
        //blockID = player.worldObj.getBlock((int) px+searchGrid[i][0],(int) py+searchGrid[i][1]  ,(int) pz+searchGrid[i][2]); // mc1.7.10
        if ( (blockID != Blocks.BED ) 
            &&      canSpawnInsideBlock(player, px+searchGrid[i][0],      py+searchGrid[i][1]  ,      pz+searchGrid[i][2])  // at feet
            &&      canSpawnInsideBlock(player, px+searchGrid[i][0],      py+searchGrid[i][1]+1,      pz+searchGrid[i][2])  // at head
            &&    (!canSpawnInsideBlock(player, px+searchGrid[i][0],      py+searchGrid[i][1]-1,      pz+searchGrid[i][2])) // under feet
           ) 
        { // then its a good location to change the bed location to
          px=px+searchGrid[i][0];
          py=py+searchGrid[i][1];
          pz=pz+searchGrid[i][2];
          spawnWasBlocked = true;
          break;
        }
      }
    }
    
    // Don't spawn inside blocks check 
    while ( ( py < player.worldObj.getActualHeight() ) 
        &&  ( (!SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) || (!SpawnCommands.canSpawnInsideBlock(player, px, py + 1, pz)) )
          ) 
    { py++;
      spawnWasBlocked = true;
    }
    
    // falling check
    while ( ( py > 1 ) 
        &&  ( (SpawnCommands.canSpawnInsideBlock(player, px, py - 1, pz)) && (SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) )
          ) 
    { py--;
      spawnWasBlocked = true;
    }
    
    // fix bed location
    if (spawnWasBlocked) { player.setSpawnChunk(new BlockPos(px,py,pz), true, bedDim); } // mc1.8
    //if (spawnWasBlocked) { player.setSpawnChunk(new ChunkCoordinates((int)px, (int)py, (int)pz), true, bedDim); } // mc1.7.10
    
    // lava under feet check
    // mc1.8
    BlockPos posBelowFeet = new BlockPos(px, (py-1), pz);
    if ((py>1) && ( ( player.worldObj.getBlockState(posBelowFeet).getBlock() == Blocks.LAVA) || ( player.worldObj.getBlockState(posBelowFeet).getBlock() == Blocks.FLOWING_LAVA) )) {
      player.worldObj.setBlockState(posBelowFeet, Blocks.COBBLESTONE.getDefaultState());
    }
    // mc1.7.10
    //if ((py>1) && ( ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.lava) || ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.flowing_lava) )) {
    //  player.worldObj.setBlock((int)px, (int)(py-1), (int)pz, Blocks.cobblestone);
    //}
    
    // actually go there now
    // mc1.9.4 : 
    // is there any way to combine the following 2 commands for simplicity? both are necessary at this time.
    //((EntityPlayerMP) player).setLocationAndAngles(px + .5d, py, pz + .5d, pyaw, ppitch); // sets angeles but does nothing client side
    //((EntityPlayerMP) player).setPositionAndUpdate(px + .5d, py, pz + .5d); // moves the player BUT doesnt set pitch and yaw.  does render the angels from above setLocationAndAngles
    Do.TeleportPlayerTo(player, px, py, pz);
    
    //player.setPositionAndUpdate(px + .5, py, pz + .5); // beds don't use yaw and pitch // mc1.9
    return;
  }
  
  public static void commandSpawn(EntityPlayer player, String[] params) {
    World world = player.worldObj; 
    if ( world.isRemote ) { return; }
    loadCommandPermissionsConfig();
    loadMiscConfig();
    
    if (params.length > 1) { showHelp(player); return; }
    
    if (params.length == 1) 
    { if (params[0].equalsIgnoreCase("help")) { SpawnCommands.showHelp(player); return; } }

    // location
    double  px;
    double  py;
    double  pz;
    Float   pyaw;
    Float   ppitch;
    int pdim;
    
    boolean isSharedName = false;
    String locationName = "";
      
    if (params.length == 1) {
      // validate named location name format. all letters and numbers and minus
      if ( params[0].substring(0,1).equals("@") ) { // if it starts with "@"
        isSharedName = true;
        params[0] = params[0].substring(1); // remove the "@"
        if ( params[0].length() == 0 ) { return; }
      }
      locationName = params[0];
      locationName = locationName.replaceAll("[^a-zA-Z0-9\\-]","");
      if ( ! locationName.contentEquals(params[0]) ) { Do.Say(player,"Named locations must use only numbers, letters, and '-'"); return; }
      locationName = locationName.toLowerCase();
      
      if ( ! isSharedName ) { // if we have a name and its not a shared name then permissions for spawnName
        // "no" and "op" and "enable" are handled here. "all" is the default.
        if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawnName").equalsIgnoreCase("no") ) 
        { Do.Say(player,"Spawning to a named location is not permitted. See config files."); return; }
        if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawnName").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
        { Do.Say(player,"Spawning to a named location is an op only command.  You are not an op."); return; }
        if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawnName").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
        { Do.Say(player,"Spawning to a named location is an advanced command and is not enabled for SpawnCommands."); return; }
      }
      
      // get current destination 
      String filePathAndName;
      if ( isSharedName ) 
      { filePathAndName = SpawnCommands.configBasePath+SpawnCommands.folderSharedNamedLocations+"/"+locationName; }
      else
      { filePathAndName = SpawnCommands.configBasePath+player.getName()+"/"+locationName; } // mc1.8
      if (! Do.fileExists(filePathAndName)) {
        Do.Say(player, "Named location '"+ (isSharedName ? "@" : "") + locationName +"' does not exist."); return;
      }
      String fileContent = Do.FileToString(filePathAndName);
      String[] fcValues = fileContent.split(","); // parse
      if ( fcValues.length != 6 ) { Do.Err(player, "Internal Error: expected 6 values: x y z yaw pitch then dimensionID. Teleport failed."); return; }
      px =   Double.parseDouble(fcValues[0]);
      py =   Double.parseDouble(fcValues[1]);
      pz =   Double.parseDouble(fcValues[2]);
      pyaw =   Float.parseFloat(fcValues[3]);
      ppitch = Float.parseFloat(fcValues[4]);
      pdim =   Integer.parseInt(fcValues[5]);
    } else { // params.length == 0 destination is main spawn
      BlockPos mainWorldSpawnPos = player.worldObj.getSpawnPoint(); // returns same pos no matter what dim you are in
      px = (double) mainWorldSpawnPos.getX();
      py = (double) mainWorldSpawnPos.getY();
      pz = (double) mainWorldSpawnPos.getZ();
      pyaw = player.rotationYaw;
      ppitch = player.rotationPitch;
      pdim = 0;       
    } // end if length == 1 or 0
    
    // actually go there now
    Do.TeleportPlayerToSC(player, px, py, pz, pyaw, ppitch, pdim);
    return;
    
  } // end commandSpawn
  
  public static void commandSpawnPlus(EntityPlayer player,String[] params) {
    World world = player.worldObj;
    if ( world.isRemote ) { return; }
    
    loadCommandPermissionsConfig();
    loadMiscConfig();
    
    if ( ( params.length == 0 ) || ( params.length == 2 ) || ( params.length == 3 ) || ( params.length > 4 ) || (params[0].equalsIgnoreCase("help")) ) 
    { SpawnCommands.showHelp(player); return; }
    
    // get player's location at start of this teleport request
    double  px = Math.round(player.posX - .5); // player's coordinates rounded down
    double  py = Math.round(player.posY - .5);
    double  pz = Math.round(player.posZ - .5);
    Float   pyaw = player.rotationYaw;
    Float   ppitch = player.rotationPitch;
    Integer pdim = world.provider.getDimension(); // mc1.9
    //Integer pdim = world.provider.getDimensionId(); // mc1.8
    //Integer pdim = world.provider.dimensionId; // mc1.7.10
    Integer originDim = pdim;
    String newGoBackLocation = px +","+ py +","+ pz +","+ pyaw +","+ ppitch +","+ pdim;
    
    // validate named location name format. all letters and numbers
    boolean isSharedName = false;
    if ( params[0].substring(0,1).equals("@") ) { // if it starts with "@"
      isSharedName = true;
      params[0] = params[0].substring(1); // remove the "@"
      if ( params[0].length() == 0 ) { return; }
    }
    String theName = params[0];
    theName = theName.replaceAll("[^a-zA-Z0-9\\-]","");
    if ( ! theName.contentEquals(params[0]) ) { Do.Say(player, "Named locations must use only numbers, letters, and '-'"); return; }
    theName = theName.toLowerCase();
    
    if ( isSharedName ) { 
      // "no" and "op" and "enable" are handled here. "all" is the default.
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+@sharedlocation").equalsIgnoreCase("no") ) 
      { Do.Say(player,"Adding a shared \u00A7e@\u00A7Rname is not permitted. See config files."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+@sharedlocation").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"Adding a shared \u00A7e@\u00A7Rname is an op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+@sharedlocation").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"Adding a shared \u00A7e@\u00A7Rname is an advanced command and is not enabled for SpawnCommands."); return; }
    }

    if ( (theName.equalsIgnoreCase("enable") || theName.equalsIgnoreCase("disable") || theName.equalsIgnoreCase("enabled") || theName.equalsIgnoreCase("disabled")) ) { 
      // "no" and "op" and "enable" are handled here. "all" is the default.
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+enable").equalsIgnoreCase("no") ) 
      { Do.Say(player,"The sub-command \"/"+Do.getConfigArrayValue(SpawnCommands.commandNames ,"spawn+")+" \u00A7eenable\u00A7R|\u00A7edisable\u00A7R\" is not permitted. See config files."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+enable").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"The sub-command \"/"+Do.getConfigArrayValue(SpawnCommands.commandNames ,"spawn+")+" \u00A7eenable\u00A7R|\u00A7edisable\u00A7R\" is an op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+enable").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"The sub-command \"/"+Do.getConfigArrayValue(SpawnCommands.commandNames ,"spawn+")+" \u00A7eenable\u00A7R|\u00A7edisable\u00A7R\" is an advanced command and is not enabled for SpawnCommands."); return; }
    }
    
    // /spawn+ enable
    if ( (theName.equalsIgnoreCase("enable") || theName.equalsIgnoreCase("enabled")) ) {
      if ( SpawnCommands.advancedCommandsAreEnabled() ) { Do.Say(player, "Advanced spawn commands are already enabled"); return; }
      SpawnCommands.miscConfigArray = Do.setConfigArrayValue(SpawnCommands.miscConfigArray,"AdvancedCommandsEnabled","true");
      if (! SpawnCommands.saveMiscConfig() ) { Do.Err(player,"Could not save changes to misc config file."); return; }
      Do.Say(player, "Advanced commands are now \u00A7eEnabled\u00A7r for SpawnCommands.");
      return; 
    }
    
    // /spawn+ disable 
    if ( (theName.equalsIgnoreCase("disable") || theName.equalsIgnoreCase("disabled")) ) {
      if ( ! SpawnCommands.advancedCommandsAreEnabled() ) { Do.Say(player, "Advanced spawn commands are already disabled"); return; }
      SpawnCommands.miscConfigArray = Do.setConfigArrayValue(SpawnCommands.miscConfigArray,"AdvancedCommandsEnabled","false");
      if (! SpawnCommands.saveMiscConfig() ) { Do.Err(player,"Could not save changes to misc config file."); return; }
      Do.Say(player, "Advanced commands are now \u00A7eDisabled\u00A7r for SpawnCommands.");
      return; 
    }
    
    // manually specified coordinates
    double parX = 0;
    double parY = 0;
    double parZ = 0;
    if ( params.length == 4 ) {
      
      // command permissions. "no" and "op" and "enable" are handled here. "all" is the default.
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+NameWithCoordinates").equalsIgnoreCase("no") ) 
      { Do.Say(player,"Creating a named location with coordinates is disabled."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+NameWithCoordinates").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"Creating a named location with coordinates is an op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+NameWithCoordinates").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"Creating a named location with coordinates is an advanced spawn command and is not enabled."); return; }
      
      boolean parXYZerror = false;
      try { parX = Double.parseDouble(params[1]); } catch  (NumberFormatException e) { Do.Say(player, "X '"+params[1]+"' is not a number."); parXYZerror = true; }
      try { parY = Double.parseDouble(params[2]); } catch  (NumberFormatException e) { Do.Say(player, "Y '"+params[2]+"' is not a number."); parXYZerror = true; }
      try { parZ = Double.parseDouble(params[3]); } catch  (NumberFormatException e) { Do.Say(player, "Z '"+params[3]+"' is not a number."); parXYZerror = true; }
      if ( parXYZerror ) { return; }
    }
    
    if (( ! isSharedName ) && ( params.length == 1)) { // /spawn+ name 
      // "no" and "op" and "enable" are handled here. "all" is the default.
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+name").equalsIgnoreCase("no") ) 
      { Do.Say(player,"Adding a named location is not permitted. See config files."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+name").equalsIgnoreCase("op") && (! Do.IsOp(player)) ) 
      { Do.Say(player,"Adding a named location is an op only command.  You are not an op."); return; }
      if ( Do.getConfigArrayValue(SpawnCommands.commandPermissions, "spawn+name").equalsIgnoreCase("enabled") && (! SpawnCommands.advancedCommandsAreEnabled() ) ) 
      { Do.Say(player,"Adding a named location is an advanced command and is not enabled for SpawnCommands."); return; }
    }
    
    // set the location
    String coordinateSource = "";
    
    if ( params.length == 4 ) { // they specified coordinates
      px = Math.round(parX - .5);
      py = Math.round(parY - .5);
      pz = Math.round(parZ - .5);
      coordinateSource = "Using ( x"+(int)px+", y"+(int)py+", z"+(int)pz+" ) in this dimension.";
      
      //when specifying coordinates do create a go back location
      //but when only a name we should still be right here so no go back location to store
      // write the location of the next go back command
      if (! Do.StringToFile( SpawnCommands.configBasePath + player.getName() + "/_" + player.getName() +"_GoBackLocation", newGoBackLocation ) ) // mc1.8
      { Do.Err(player,"CommandSpawn: Could not create the file that stores the location to go back to."); return; }
      // above error prevents creation of the new named location
      
    } else { // they only specified a name
      px = Math.round(player.posX - .5); // player's coordinates rounded down
      py = Math.round(player.posY - .5);
      pz = Math.round(player.posZ - .5);
      coordinateSource = "Using your current coordinates.";
    }

    pyaw = player.rotationYaw;
    ppitch = player.rotationPitch;
    pdim = player.dimension; // mc 1.9.4
    //pdim = player.worldObj.provider.getDimension(); // mc1.9 // player's current dimension
    //pdim = player.worldObj.provider.getDimensionId(); // mc1.8 // player's current dimension
    //pdim = player.worldObj.provider.dimensionId; // mc1.7.10 // player's current dimension

    // Don't spawn inside blocks check   
    while ( ( py < player.worldObj.getActualHeight() ) 
        &&  ( (!SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) || (!SpawnCommands.canSpawnInsideBlock(player, px, py + 1, pz)) )
          ) 
    { py++; }
    // falling check
    while ( ( py > 1 ) 
        &&  ( (SpawnCommands.canSpawnInsideBlock(player, px, py - 1, pz)) && (SpawnCommands.canSpawnInsideBlock(player, px, py, pz)) )
          ) 
    { py--; }
    // lava under feet check
    // mc1.8
    BlockPos posBelowFeet = new BlockPos(px, (py-1), pz);
    if ((py>1) && ( ( player.worldObj.getBlockState(posBelowFeet).getBlock() == Blocks.LAVA) || ( player.worldObj.getBlockState(posBelowFeet).getBlock() == Blocks.FLOWING_LAVA) )) {
      player.worldObj.setBlockState(posBelowFeet, Blocks.COBBLESTONE.getDefaultState());
    }
    // mc1.7.10
    //if ((py>1) && ( ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.lava) || ( player.worldObj.getBlock((int)px, (int)(py-1), (int)pz) == Blocks.flowing_lava) )) {
    //  player.worldObj.setBlock((int)px, (int)(py-1), (int)pz, Blocks.cobblestone);
    //}
    
    // identify the named locations folder and file
    String filePath;
    if ( isSharedName ) 
    { filePath = SpawnCommands.configBasePath+SpawnCommands.folderSharedNamedLocations+"/"; }
    else
    { filePath = SpawnCommands.configBasePath+player.getName()+"/"; } // mc1.8
    String filePathAndName = filePath+theName;
    Do.folderMake(filePath+".");
    
    // Limit The Number Of Named Locations Per Player
    loadMiscConfig();
    if ( Do.getConfigArrayValueBoolean(SpawnCommands.miscConfigArray,"LimitTheNumberOfNamedLocationsPerPlayer") && (! isSharedName)) {
      int theLimit = Do.getConfigArrayValueInteger(SpawnCommands.miscConfigArray,"TheNamedLocationsLimitNumber");
      // count their named locations and detect if replacing one of them.
      int isReplacementName = 0;
      File thefolder = new File(filePath+".");
      String[] filesInDir = thefolder.list();
      int numberOfNamedLocations = 0;
      if ( filesInDir != null ) { 
        for ( int i=0; i<filesInDir.length; i++ ) {
          if ( filesInDir[i].equalsIgnoreCase(theName) ) { isReplacementName = 1; }
          if ( ! filesInDir[i].startsWith("_",0) ) { numberOfNamedLocations++; }
        }
      }
      if ( (numberOfNamedLocations - isReplacementName ) >= theLimit ) 
      { Do.Say(player,"Sorry, that would create more than the "+theLimit+" allowed named locations per player. Try removing one first?"); return; }
    }
    
    // write the named location file
    if (! Do.StringToFile(filePathAndName, px +","+ py +","+ pz +","+ pyaw +","+ ppitch +","+ pdim) ) 
    { Do.Err(player,"Named location NOT set. Check folder "+SpawnCommands.folderSharedNamedLocations); return; }
    
    Do.Say(player,coordinateSource); // from specified coords or from where player is
    
    // actually go there
    // both the below lines are required for mc1.9.4
    //((EntityPlayerMP) player).setLocationAndAngles(px + .5d, py, pz + .5d, pyaw, ppitch); // sets angeles but does nothing client side
    //((EntityPlayerMP) player).setPositionAndUpdate(px + .5d, py, pz + .5d); // moves the player BUT doesnt set pitch and yaw.  does render the angels from above setLocationAndAngles
    Do.TeleportPlayerTo(player, px, py, pz, pyaw, ppitch, pdim); // cannot use SC versions of function. Not using the go back when creating a new named location
    
    Do.Say(player, (isSharedName ? "@" : "") + theName + " location is now set.");
    
    return;
    
  } // end commandSpawnPlus
  
}
